/***************************************************************************
  FILE: TAgentJS.java, TomaszMajor-10Oct98
  PURPOSE: TAgent for JavaScript
  $Id: TAgentJS.java,v 1.11 1999/10/28 17:13:10 remek Exp $
  Component: Plugin, Interfaces
***************************************************************************/


package webwisdom.tango;


import java.util.Vector;

import webwisdom.tango.structures.LocalBase;

import netscape.javascript.JSObject;
import netscape.plugin.Plugin;
//import netscape.javascript.JSException;


/**Tango system proxy to be used in JavaScript applications.
   <br>Only methods ending with "JS" are designed to be called from scripts.
   The functionality corresponds to the one of <tt>TAgent</tt>.
   *@see TAgent*/
public final class TAgentJS extends TAgent
  implements TDataListener, TMasterListener, TParticipantsListener, TCrossDataListener
{
    private static final String CL="TAgentJS";


    protected JSObject script=null;

    protected Vector dataListScr=new Vector();
    protected Vector mastListScr=new Vector();
    protected Vector partListScr=new Vector();
    protected Vector crossListScr=new Vector();

    private boolean dbgFlag=false;


    /**
     *@param scr script - main application window
     which will receive focus and will closed on termination*/
    private TAgentJS(TLAgent a,JSObject scr)
    {
      super(a);
      script=scr;
    }// TAgentJS ___________________________________________________________


    /**Sets or unsets debug mode.
       If TAgentJS operates in debug mode
       then every time JavaScript interface functions are used
       it prints to System.out the name of the method called
       and the arguments passed.
       Arguments are printed in the Java, not JavaScript form.
     *@param debug whether TAgentJS should operate in debug mode*/
    public void setDbgModeJS(Boolean debug)
    {
      dbgFlag=debug.booleanValue();
      System.out.println(CL+".setDbgModeJS("+dbgFlag+")");
    }// setDbgModeJS _______________________________________________________


    /**
     *@return Tango version
     *@see TAgent#getVersion*/
    public JSObject getVersionJS()
    {
      String v=super.getVersion();
      if(dbgFlag)
	System.out.println(CL+".getVersionJS()="+v);
      //REMEK:  this used to be commented out and return null
      Plugin lb=LocalBase.getLocalBase();
      if(lb==null)
	  return null;
      JSObject s=lb.getWindow();
      return (JSObject)s.eval("new String('"+v+"');");
    }// getVersionJS _______________________________________________________


    /**Deregisters this application.*/
    public void exitJS()
    {
      if(dbgFlag)
	System.out.println(CL+".exitJS()");
      exit();
    }// exitJS _____________________________________________________________


    /**Adds a recipient of the session data.
     *@param l script/window receiving application messages.
     The given window must implement <tt>Tango_receive()</tt> method.
     *@see #sendJS*/
    public void addTDataListenerJS(JSObject l)
    {
      if(dbgFlag)
	System.out.println(CL+".addTDataListenerJS("+l+")");
      dataListScr.removeElement(l);
      dataListScr.addElement(l);
      addTDataListener(this);
    }// addTDataListenerJS _________________________________________________


    /**Stops receiving private data by the given script/window.
       This function reverses the effect of <tt>addTDataListener()</tt>
       with the same argument.
       A listener can be removed and added as often as necessary.
       *@param l listener script/window
       *@see #addTDataListenerJS*/
    public void removeTDataListenerJS(JSObject l)
    {
      if(dbgFlag)
	System.out.println(CL+".removeTDataListenerJS("+l+")");
      dataListScr.removeElement(l);
      if(dataListScr.size()==0)
	removeTDataListener(this);
    }// removeTDataListenerJS ______________________________________________


    /**Adds a recipient of the Tango system notifications.
     *@param l listener script/window - this script has to implement
     the control listener interface:
     <ul>
     <li><tt>Tango_masterChanged(isMaster,masterName);</tt>
     <li><tt>Tango_participantJoined(participantName);</tt>
     <li><tt>Tango_participantLeft(participantName).</tt>
     </ul>
     *@deprecated use addTMasterListenerJS() or addTParticipantsListenerJS()*/
    public void addTControlListenerJS(JSObject l)
    {
      addTMasterListenerJS(l);
      addTParticipantsListenerJS(l);
    }// addTControlListenerJS ______________________________________________


    /**Stops receiving control data by the given script/window.
       This function reverses the effect of <tt>addTControlListener()</tt>
       with the same argument.
       A listener can be removed and added as often as necessary.
       *@param l listener script/window
       *@see #addTControlListenerJS
       *@deprecated*/
    public void removeTControlListenerJS(JSObject l)
    {
      removeTMasterListenerJS(l);
      removeTParticipantsListenerJS(l);
    }// removeTControlListenerJS ___________________________________________


    /**Adds a recipient of master notifications.
     *@param l listener script/window - this script has to implement
     the master listener interface:
     <ul>
     <li><tt>Tango_masterChanged(isMaster,masterName);</tt>
     </ul>*/
    public void addTMasterListenerJS(JSObject l)
    {
      if(dbgFlag)
	System.out.println(CL+".addTMasterListenerJS("+l+")");
      mastListScr.removeElement(l);
      mastListScr.addElement(l);
      addTMasterListener(this);
    }// addTMasterListenerJS _______________________________________________


    /**Stops receiving master notifications by the given script/window.
       This function reverses the effect of <tt>addTMasterListener()</tt>
       with the same argument.
       A listener can be removed and added as often as necessary.
       *@param l listener script/window
       *@see #addTMasterListenerJS*/
    public void removeTMasterListenerJS(JSObject l)
    {
      if(dbgFlag)
	System.out.println(CL+".removeTMasterListenerJS("+l+")");
      mastListScr.removeElement(l);
      if(mastListScr.size()==0)
	removeTMasterListener(this);
    }// removeTMasterListenerJS ____________________________________________


    /**Adds a recipient of system notifications about participants.
     *@param l listener script/window - this script has to implement
     the participants listener interface:
     <ul>
     <li><tt>Tango_participantJoined(participantName);</tt>
     <li><tt>Tango_participantLeft(participantName).</tt>
     </ul>*/
    public void addTParticipantsListenerJS(JSObject l)
    {
      if(dbgFlag)
	System.out.println(CL+".addTParticipantsListenerJS("+l+")");
      partListScr.removeElement(l);
      partListScr.addElement(l);
      addTParticipantsListener(this);
    }// addTParticipantsListenerJS _________________________________________


    /**Stops receiving notifications about participants
       by the given script/window.
       This function reverses the effect of <tt>addTParticipantsListener()</tt>
       with the same argument.
       A listener can be removed and added as often as necessary.
       *@param l listener script/window
       *@see #addTParticipantsListenerJS*/
    public void removeTParticipantsListenerJS(JSObject l)
    {
      if(dbgFlag)
	System.out.println(CL+".removeTParticipantsListenerJS("+l+")");
      partListScr.removeElement(l);
      if(partListScr.size()==0)
	removeTParticipantsListener(this);
    }// removeTParticipantsListenerJS ______________________________________


    /**Adds a recipient of data from different sessions.
     *@param l script/window receiving messages.
     The given window must implement <tt>Tango_receive()</tt> method.
     *@see #crossSendJS*/
    public void addTCrossDataListenerJS(JSObject l)
    {
      if(dbgFlag)
	System.out.println(CL+".addTCrossDataListenerJS("+l+")");
      crossListScr.removeElement(l);
      crossListScr.addElement(l);
      addTCrossDataListener(this);
    }// addTCrossDataListenerJS ____________________________________________


    /**Stops receiving data from other sessions.
       This function reverses the effect of <tt>addTCrossDataListener()</tt>
       with the same argument.
       A listener can be removed and added as often as necessary.
       *@param l listener script/window
       *@see #addTCrossDataListenerJS*/
    public void removeTCrossDataListenerJS(JSObject l)
    {
      if(dbgFlag)
	System.out.println(CL+".removeTCrossDataListenerJS("+l+")");
      crossListScr.removeElement(l);
      if(crossListScr.size()==0)
	removeTCrossDataListener(this);
    }// removeTCrossDataListenerJS _________________________________________


    /**Sends string messages.
     *@param data string to be sent*/
    public void sendJS(String data)
    {
      if(dbgFlag)
	System.out.println(CL+".sendJS("+data+")");
      send(stringToBytes(data));
    }// sendJS _____________________________________________________________


    /**Sends string messages to selected users.
     *@param data string to be sent
     *@param part participants which are to receive the message
     *@see #getParticipantNamesJS*/
    public void selectiveSendJS(JSObject part,String data)
    {
      //System.out.println(CL+".selectiveSendJS("+part+","+data+")");
      Object o=part.getMember("length");
      int len=((Number)o).intValue();
      String[] p=new String[len];
      for(int i=0;i<len;i++)
      {
	Object s=part.getSlot(i);
	//System.out.println(CL+".selectiveSendJS(): part["+i+"]="+s);
	p[i]=(String)s;
      }
      if(dbgFlag)
	System.out.println(CL+".selectiveSendJS("+arrayToString(p)+","+data+")");
      selectiveSend(p,stringToBytes(data));
    }// selectiveSendJS ____________________________________________________


    /**Sends a message to another session.
     *@param sess recipient session
     *@param data string to be sent*/
    public void crossSendJS(String sess,String data)
    {
      if(dbgFlag)
	System.out.println(CL+".crossSendJS("+sess+","+data+")");
      crossSend(sess,stringToBytes(data));
    }// crossSendJS ________________________________________________________


    public void receive(byte data[])
    {
      for(int i=0;i<dataListScr.size();i++)
      {
	JSObject w=(JSObject)dataListScr.elementAt(i);
	try
	{
	  Object eval=w.eval("setTimeout(\'Tango_receive(\""+bytesToString(data)+"\")\',0);");
	}
	catch(Exception e)
	{
	  e.printStackTrace();
	}

// 	Object[] args=new Object[1];
// 	args[0]=script.eval("new String('"+bytesToString(data)+"');");
// 	((JSObject)dataListScr.elementAt(i)).call("Tango_receive",args);
      }
    }// receive ____________________________________________________________


    public void crossReceive(String sess,byte data[])
    {
      for(int i=0;i<crossListScr.size();i++)
      {
	JSObject w=(JSObject)crossListScr.elementAt(i);
	try
	{
	  Object eval=w.eval("setTimeout(\'Tango_crossReceive(\""+sess+"\",\""+bytesToString(data)+"\")\',0);");
	}
	catch(Exception e)
	{
	  e.printStackTrace();
	}
      }
    }// crossReceive _______________________________________________________


    public void masterChanged(boolean isMaster,String masterName)
    {
      for(int i=0;i<mastListScr.size();i++)
      {
	JSObject w=(JSObject)mastListScr.elementAt(i);
	try
	{
	  Object eval=w.eval("setTimeout(\'Tango_masterChanged("+(isMaster?"true":"false")+",\""+masterName+"\")\',0);");
	}
	catch(Exception e)
	{
	  e.printStackTrace();
	}
      }
    }// masterChanged ______________________________________________________


    public void participantJoined(String name)
    {
      for(int i=0;i<partListScr.size();i++)
      {
	JSObject w=(JSObject)partListScr.elementAt(i);
	try
	{
	  Object eval=w.eval("setTimeout(\'Tango_participantJoined(\""+name+"\")\',0);");
	}
	catch(Exception e)
	{
	  e.printStackTrace();
	}
	//System.out.println(CL+".participantJoined(): eval="+eval);
      }
    }// participantJoined __________________________________________________


    public void participantLeft(String name)
    {
      for(int i=0;i<partListScr.size();i++)
      {
	JSObject w=(JSObject)partListScr.elementAt(i);
	try
	{
	  Object eval=w.eval("setTimeout(\'Tango_participantLeft(\""+name+"\")\',0);");
	}
	catch(Exception e)
	{
	  e.printStackTrace();
	}
      }
    }// participantLeft ____________________________________________________


//     public void audioRequested(boolean r)
//     {
//       for(int i=0;i<contListScr.size();i++)
//       {
// 	Object args[]=new Object[1];
// 	args[0]=null;
// 	try
// 	{
// 	  args[0]=script.eval("new Boolean("+(r?"1":"0")+");");
// 	}
// 	catch(Exception e)
// 	{
// 	  e.printStackTrace();
// 	}
// 	((JSObject)contListScr.elementAt(i)).call("Tango_audioRequested",args);
//       }
//     }// audioRequested _____________________________________________________


    /**
     *@return String - this user name (as logged to the system)*/
    public JSObject getUserNameJS()
    {
      String name=getUserName();
      if(dbgFlag)
	System.out.println(CL+".getUserNameJS()="+name);
      Object eval=null;
      try
      {
        eval=script.eval("new String('"+name+"');");
      }
      catch(Exception e)
      {
	e.printStackTrace();
      }
      return (JSObject)eval;
    }// getUserNameJS ______________________________________________________


    /**
     *@deprecated use getParticipantsNamesJS() instead*/
    public JSObject getParticipantNamesJS()
    {
      return getParticipantsNamesJS();
    }// getParticipantNamesJS ______________________________________________


    /**
     *@return Array object - array of Strings representing names
     of the users of this session*/
    public JSObject getParticipantsNamesJS()
    {
      String[] names=getParticipantsNames();
      if(dbgFlag)
	System.out.println(CL+".getParticipantsNamesJS()="+arrayToString(names));
      Object eval=null;
      try
      {
	eval=script.eval("new Array("+arrayToString(names,"'","','","'")+");");
      }
      catch(Exception e)
      {
	e.printStackTrace();
      }
      //System.out.println(CL+".getParticipantNamesJS(): eval="+eval);
      return (JSObject)eval;
    }// getParticipantsNamesJS ______________________________________________


    /**
     *@return the name of the master of this session*/
    public JSObject getMasterNameJS()
    {
      String name=getMasterName();
      if(dbgFlag)
	System.out.println(CL+".getMasterNameJS()="+name);
      Object eval=null;
      try
      {
	eval=script.eval("new String('"+name+"');");
      }
      catch(Exception e)
      {
	e.printStackTrace();
      }
      return (JSObject)eval;
    }// getMasterNameJS ____________________________________________________


    /**
     *@return Boolean object - true if this user
     is the master of this session*/
    public JSObject isMasterJS()
    {
      boolean isM=isMaster();
      if(dbgFlag)
	System.out.println(CL+".isMasterJS()="+isM);
      Object eval=null;
      try
      {
	eval=script.eval("new Boolean("+(isM?"1":"0")+");");
      }
      catch(Exception e)
      {
	e.printStackTrace();
      }
      return (JSObject)eval;
    }// isMasterJS _________________________________________________________


    /**
     *@return Boolean object - true if the audio resources
     can be used by this session*/
    public JSObject isAudioAvailableJS()
    {
      boolean isA=isAudioAvailable();
      if(dbgFlag)
	System.out.println(CL+".isAudioAvailableJS()="+isA);
      Object eval=null;
      try
      {
	eval=script.eval("new Boolean("+(isA?"1":"0")+");");
      }
      catch(Exception e)
      {
	e.printStackTrace();
      }
      return (JSObject)eval;
    }// isAudioAvailableJS _________________________________________________


    /**
     *@return Array object - array of Strings representing names
     of all active session*/
    public JSObject getJoinedSessionsNamesJS()
    {
      String[] names=getJoinedSessionsNames();
      if(dbgFlag)
	System.out.println(CL+".getJoinedSessionsNamesJS()="
			   +arrayToString(names));
      Object eval=null;
      try
      {
	eval=script.eval("new Array("+arrayToString(names,"'","','","'")+");");
      }
      catch(Exception e)
      {
	e.printStackTrace();
      }
      //System.out.println(CL+".getJoinedSessionsNamesJS(): eval="+eval);
      return (JSObject)eval;
    }// getJoinedSessionsNamesJS ___________________________________________


    private String bytesToString(byte data[])
    {
      return new String(data);
    }// bytesToString ______________________________________________________


    private byte[] stringToBytes(String data)
    {
      return data.getBytes();
    }// stringToBytes ______________________________________________________


    private static String arrayToString(Object[] ss)
    {
      return arrayToString(ss,"{",",","}");
    }// arrayToString ______________________________________________________


    /**
       *@param ss list to be printed
       *@param beg,end opening and closing braces
       *@param sep list separator
       *@return listing of the input array as a set
       (<beg><ss[0]><sep><ss[1]><sep>...<end>)*/
    private static String arrayToString(Object[] ss,
					String beg,String sep,String end)
    {
      int n;
      if(ss!=null)
	n=ss.length;
      else
	n=6;
      StringBuffer s=new StringBuffer(n*10);


      if(beg!=null)
	s.append(beg);

      if(ss!=null)
      {
	for(int i=0;i<n;i++)
	{
	  s.append(ss[i].toString());
	  if(i<(n-1))
	    s.append(sep);
	}
      }

      if(end!=null)
	s.append(end);


      return s.toString();
    }// arrayToString ______________________________________________________


    /**Registers application and creates handle on a Tango agent.
     *@param scr script/window to be registered - the top of this window
     will be closed upon the application termination*/
    public static TAgentJS createTAgentJS(JSObject scr)
    {
      try
      {
	return new TAgentJS(new TLAgentApplet(scr),scr);
      }
      catch(TangoException e)
      {
	return null;
      }
    }// createTAgentJS _____________________________________________________


    /**Registers application and creates handle on a fake Tango agent.
     *@param scr script/window to be registered - the top of this window
     will be closed upon the application termination
     *@param host,port fake server location*/
    public static TAgentJS createTAgentFakeJS(JSObject scr,
					      String host,Number port)
    {
      try
      {
	return new TAgentJS(new webwisdom.tango.fake.TLAgentFake(host,port.intValue()),scr);
      }
      catch(TangoException e)
      {
	return null;
      }
    }// createTAgentJS _____________________________________________________


    public String toString()
    {
      return CL+"["+super.toString()+"]";
    }// toString ___________________________________________________________


}// TAgentJS ///////////////////////////////////////////////////////////////


