/***************************************************************************
  FILE: TLAgentApplet.java, TomaszMajor,PiotrSokolowski-10Apr98
  PURPOSE: interface component for applets
  $Id: TLAgentApplet.java,v 1.21 1999/09/08 18:06:27 toma Exp $
  Component: Plugin, Interfaces
***************************************************************************/


package webwisdom.tango;


import java.applet.Applet;
import java.util.Vector;
import java.util.StringTokenizer;
import java.awt.*;

import webwisdom.tango.consts.CAConsts;
import webwisdom.tango.consts.Const;
import webwisdom.tango.messages.AppEventMessage;
import webwisdom.tango.structures.AppletHandle;
import webwisdom.tango.structures.LocalBase;
import webwisdom.tango.interfaces.AppletBaseInterf;

import netscape.javascript.JSObject;


/**Tango system proxy for applets.
   This class may be used to create TAgent.
   *@see TAgent*/
public final class TLAgentApplet
  implements TLAgent, AppletBaseInterf
{
    private static final String CL="TLAgentApplet";


    private boolean inFrame=false;
    private JSObject window=null;
    private int sessionId=-1,applicationId=-1;

    private AppletHandle handle=null;

    private AemMsgHandler aemMsgHandler;
    private QueueHandler msgQueue;


    /**Wrapper for Registrar.registerApplet()*/
    private static AppletHandle register(AppletBaseInterf base,
					 RegParam reg)
      throws TangoException
    {
      //System.out.println(CL+".register(ABI,"+reg+")");
      try
      {
	AppletHandle h;
	if(!reg.inf)
	  h=LocalBase.getLocalBase().reg.registerApplet(base,reg.aid,reg.sid,reg.at);
	else
	  h=LocalBase.getLocalBase().reg.registerFrameApplet(base,reg.aid,reg.sid,reg.at,reg.pre);
	if(h==null)
	  throw new TangoException(CL+": registration failed: Registrar");
	return h;
      }
      catch(UnsatisfiedLinkError e)
      {
	//e.printStackTrace();
	throw new TangoException(CL+": missing library: "+e.getMessage());	
      }
      catch(NoClassDefFoundError e)
      {
	//e.printStackTrace();
	throw new TangoException(CL+": missing classes: "+e.getMessage());	
      }
      catch(TangoException e)
      {
	//e.printStackTrace();
	throw e;
      }
      catch(Exception e)
      {
	e.printStackTrace();
	throw new TangoException(CL+": registration failed: "+e.getMessage());
      }
    }// register ___________________________________________________________


    /**Initialization common for all constructors.*/
    private void init(RegParam reg) throws TangoException
    {
      //System.out.println(CL+".init("+reg+")");

      sessionId=reg.sid;
      applicationId=reg.aid;
      inFrame=reg.inf;


      String name=CL+"["+getSessionId()+"].msgQueue";
      aemMsgHandler=new AemMsgHandler();
      msgQueue=new QueueHandler(name,aemMsgHandler);


      handle=register(this,reg);
    }// init _______________________________________________________________



    private static final String COVERLAYER="cover";

    /**Hides the layer that originally covers unloaded applet.
     *@see #COVERLAYER*/
    private static void uncover(JSObject s)
    {
      try
      {
	Object d=s.getMember("document");
	//System.out.println(CL+".uncover(): d="+d);
	Object c=((JSObject)d).getMember(COVERLAYER);
	//System.out.println(CL+".uncover(): c="+c);
	if((c!=null)&&(c instanceof JSObject))
	{
	  ((JSObject)c).setMember("visibility","hide");
	}

	//Object eval=s.eval("document."+COVERLAYER+".visibility='hide'");
	//System.out.println(CL+".uncover(): eval="+eval);
      }
      catch(Exception e)
      {
	System.err.println(CL+".uncover(): layer \""+COVERLAYER+"\" not found");
	//e.printStackTrace();
      }
    }// uncover ____________________________________________________________


    /**Registers an applet in the Tango system.
     *@param app applet to be registered
     *@exception TangoException if cannot get reference to Tango*/
    public TLAgentApplet(Applet app) throws TangoException
    {
      if(app==null)
	throw new IllegalArgumentException(CL+": null applet");
      try
      {
	window=(JSObject)JSObject.getWindow(app);
      }
      catch(Exception e)
      {
	e.printStackTrace();
      }

      init(new RegParam(window));

      uncover(window);
    }// TLAgentApplet ______________________________________________________


    /**Registers an applet in the Tango system.
     *@param win Java Script Window object opened by Session Manager
     *@exception TangoException if cannot get reference to Tango*/
    public TLAgentApplet(JSObject win) throws TangoException
    {
      if(win==null)
	throw new IllegalArgumentException(CL+": null window");
      window=win;

      init(new RegParam(window));

      uncover(window);
    }// TLAgentApplet ______________________________________________________


    /**Registers an applet in the Tango system.
     *@param win Java Script Window object used for focusing and closing
     *@param reg string with reg params
     *@exception TangoException if cannot get reference to Tango*/
    public TLAgentApplet(JSObject win,String reg)
      throws TangoException
    {
      window=win;

      init(new RegParam(reg));
    }// TLAgentApplet ______________________________________________________


//     private void setOnUnloadHandler(Applet app)
//     {
//       //System.out.println(CL+".setOnUnloadHandler()");
//       JSObject win=(JSObject)JSObject.getWindow(app);
//       Object e=win.eval("var Tango_onUnloadProcessor=null;");
//       e=win.eval("function Tango_setOnUnloadProcessor(p){Tango_onUnloadProcessor=p;}");
//       {
// 	Object args[]=new Object[1];
// 	args[0]=this;
// 	e=win.call("Tango_setOnUnloadProcessor",args);
//       }
//       e=win.eval("function Tango_onUnloadHandler(){Tango_onUnloadProcessor.onUnloadHandlerJS();}");
//       e=win.eval("window.onunload=Tango_onUnloadHandler;");
//       //System.out.println(CL+".setOnUnloadHandler(): eval="+eval);
//     }// setOnUnloadHandler _________________________________________________


    public void exit()
    {
      //System.out.println(CL+".exit()");
      if(handle!=null)
      {
	handle.remove();
	handle=null;
      }
      msgQueue.stop();
      //System.out.println(CL+".exit().");
    }// exit _______________________________________________________________


    public void setTLListener(TLListener tl)
    {
      //System.out.println(CL+".setTLListener("+tl+")");
      aemMsgHandler.setListener(tl);
      if(!msgQueue.isRunning())
	(new Thread(msgQueue,msgQueue.getName())).start();
    }// setTLListener ______________________________________________________


    public final void send(AppEventMessage m)
    {
      //System.out.println(CL+".send("+m+")");
      handle.sendAppToOther(m);
    }// send _______________________________________________________________


    public void send(int userlist[],AppEventMessage m)
    {
      //System.out.println(CL+".send(["+userlist.length+"],"+m+")");
      handle.sendAppToOther(userlist,m);
    }// send _______________________________________________________________


    public void receive(AppEventMessage m)
    {
      //System.out.println(CL+".receive("+m+")");
      msgQueue.add(m);
    }// receive ____________________________________________________________


    public Object getJSWindow()
    {
      //System.out.println(CL+".getJSWindow()");
      if(inFrame)
	return window;
      else
	return (JSObject)window.getMember("top");
    }// getJSWindow ________________________________________________________
    

    public int getSessionId()
    {
      return sessionId;
    }// getSessionId _______________________________________________________


    public int getApplicationId()
    {
      return applicationId;
    }// getApplicationId ___________________________________________________


    public String toString()
    {
      return CL+"[sid="+getSessionId()+",w=Window@"+window.hashCode()+
	",inF="+inFrame+",h="+handle+"]";
    }// toString ___________________________________________________________


}// TLAgentApplet //////////////////////////////////////////////////////////

