/***************************************************************************
  FILE: TLAgentApplication.java, TomaszMajor-10Dec98
  PURPOSE: interface component for applications
  $Id: TLAgentApplication.java,v 1.6 1999/09/08 16:14:10 toma Exp $
  Component: Plugin, Interfaces
***************************************************************************/


package webwisdom.tango;


import java.net.*;
import java.io.*;

import webwisdom.tango.consts.Const;
import webwisdom.tango.consts.CAConsts;
import webwisdom.tango.messages.AppEventMessage;
import webwisdom.tango.messages.CAppAnswerEventMessage;


class TLAgentApplicationReceiver extends Thread
{
    private static final String CL="TLAgentApplicationReceiver";


    private String name;
    private DataInputStream inStream;
    private TLAgentApplication agent;


    public TLAgentApplicationReceiver(String n,
				      DataInputStream is,TLAgentApplication a)
    {
      super(n);
      name=n;
      inStream=is;
      agent=a;
    }// TLAgentApplicationReceiver _________________________________________


    public void run()
    {
      try
      {
	do
	{
	  System.out.println(name+": waiting");
	  int type=inStream.readInt();
	  int packetLen=inStream.readInt();
	  byte[] packetData=new byte[packetLen];
	  inStream.readFully(packetData);
	  AppEventMessage m=new AppEventMessage(type,packetData);
	  System.out.println(name+": processing byte["+packetData.length+"]");
	  //System.out.println(CL+": <"+new String(packetData,0));
	  agent.receive(m);
	}
	while(true);
      }
      catch(EOFException e)
      {
	//System.err.println(CL+": Server disconnected!");
	agent.exit();
      }
      catch(IOException e)
      {
	System.err.println(name+": "+e);
	e.printStackTrace();
      }
    }// run ________________________________________________________________


    public String toString()
    {
	return CL+"["+name+"]";
    }// toString ___________________________________________________________


}// TLAgentApplicationReceiver /////////////////////////////////////////////




/**Tango system proxy for applications.
   This class may be used to create TAgent.
   *@see TAgent*/
public final class TLAgentApplication implements TLAgent
{
    private static final String CL="TLAgentApplication";


    private Socket socket;
    private DataOutputStream outStream;
    private DataInputStream inStream;

    private TLAgentApplicationReceiver receiverThread;

    private AemMsgHandler aemMsgHandler;
    private QueueHandler msgQueue;

    private int sessionId=-1,applicationId=-1;


    /**Registers an application in the Tango system.
     *@param args original arguments passed to the program
     *@exception TangoException if cannot connect to Tango*/
    public TLAgentApplication(String[] args)
      throws TangoException
    {
      //System.out.println(CL+".TLAgentApplication("+args+")");

      init(new RegParam(args));

      //System.out.println(CL+".TLAgentApplication()="+this);
    }// TLAgentApplication _________________________________________________


    public TLAgentApplication(int aid,int sid,int at,int port)
      throws TangoException
    {
      //System.out.println(CL+".TLAgentApplication("+aid+","+sid+","+at+","+port+")");

      init(new RegParam(aid,sid,at,port));

      //System.out.println(CL+".TLAgentApplication()="+this);
    }// TLAgentApplication _________________________________________________


    /**Initialization common for all constructors.*/
    private void init(RegParam reg) throws TangoException
    {
      System.out.println(CL+".init("+reg+")");

      sessionId=reg.sid;
      applicationId=reg.aid;


      String name=CL+"["+getSessionId()+"].msgQueue";
      aemMsgHandler=new AemMsgHandler();
      msgQueue=new QueueHandler(name,aemMsgHandler);


      try
      {
	InetAddress host=InetAddress.getLocalHost();
	socket=new Socket(host,reg.port); 
	outStream=new DataOutputStream(socket.getOutputStream());
	inStream=new DataInputStream(socket.getInputStream());
	System.out.println(CL+".init(): connected to: "+socketToString(socket)+
			   " (localPort="+socket.getLocalPort()+")");
      }
      catch(IOException e)
      {
	throw new TangoException("port "+reg.port+" unreachable");
      }


      name=CL+"["+getSessionId()+"].rcvThr";
      receiverThread=new TLAgentApplicationReceiver(name,inStream,this);
      receiverThread.start();
      (Thread.currentThread()).setPriority(Thread.MIN_PRIORITY);
      //pt.setPriority(Math.max(pt.getPriority()-1,Thread.MIN_PRIORITY));


      register(reg);

      //System.out.println(CL+".init().");
    }// init _______________________________________________________________


    private void register(RegParam reg)
      throws TangoException //,IOException
    {
      //System.out.println(CL+".register("+reg+")");
      try
      {
	outStream.writeInt(reg.aid);
	outStream.writeInt(reg.sid);
	outStream.writeInt(reg.at);
	outStream.flush();
      }
      catch(IOException e)
      {
	throw new TangoException("I/O error: "+e.getMessage());
      }
      //System.out.println(CL+".register().");
    }// register ___________________________________________________________


    /*interface for TLAgentApplicationReceiver*/
    void receive(AppEventMessage m)
    {     
      //System.out.println(CL+".receive("+m+")");
      if(m.getType()==Const.DATA)
      {
	//System.out.println(CL+".receive("+m+"): DATA");
        msgQueue.add(m);
      }
      else if(m.getType()==Const.CONTROL)
      {
	//System.out.println(CL+".receive("+m+"): CONTROL");
	CAppAnswerEventMessage mm=new CAppAnswerEventMessage(m.getData());
	//System.out.println(CL+".receive("+m+"): mm="+mm);
        msgQueue.add(mm);
      }
      else if(m.getType()==Const.PING)
      {
	//System.out.println(CL+".receive("+m+"): PING");
	send(m);
      }
      else
      {
	;//System.err.println(CL+".receive("+m+"): unknown message type!");
      }
    }// receive ____________________________________________________________


    public void send(AppEventMessage m)
    {
      System.out.println(CL+".send("+m+")");
      try
      {
	m.send(outStream);
      }
      catch(IOException e)
      {
	System.err.println(CL+".send("+m+"): "+e);
	e.printStackTrace();
      }
    }// send _______________________________________________________________


    /**This function is not implemented*/
    public void send(int userlist[],AppEventMessage m)
    {
      //System.out.println(CL+".send("+arrayToString(userlist)+","+m+")");
      System.err.println(CL+".send(int[],AppEventMessage): not implemeted! redirecting to send(AppEventMessage)");
      send(m);
    }// send _______________________________________________________________


    public void exit()
    {
      //System.out.println(CL+".exit()");

//       try
//       {
// 	if(socket!=null)
// 	  socket.close();
//       }
//       catch(IOException e)
//       {
//       }

//       if((receiverThread!=null)&&(receiverThread.isAlive()))
// 	receiverThread.stop();

      //System.exit(0);
    }// exit _______________________________________________________________


    public int getSessionId()
    {
      return sessionId;
    }// getSessionId _______________________________________________________


    public int getApplicationId()
    {
      return applicationId;
    }// getApplicationId ___________________________________________________


    public void setTLListener(TLListener tl)
    {
      //System.out.println(CL+".setTLListener("+""+")");
      aemMsgHandler.setListener(tl);
      if(!msgQueue.isRunning())
	(new Thread(msgQueue,msgQueue.getName())).start();
      //System.out.println(CL+".setTLListener().");
    }// setTLListener ______________________________________________________


    //utility

    private static String socketToString(Socket s)
    {
      String host=s.getInetAddress().getHostName();
      int port=s.getPort();
      return host+":"+port;
    }// socketToString _____________________________________________________


    public String toString()
    {
      return CL+"[sid="+getSessionId()+","+socketToString(socket)+"]";
    }// toString ___________________________________________________________


}// TLAgentApplication /////////////////////////////////////////////////////


