//=============================================================================
//
// class DataEditor
//
// Author:			Grzegorz Lewandowski
// Created:			12/02/98
// Original class:  UserEditor
//
// Modified:		07/20/99
// By:				Hyun Kim
// Changes:			class name changed from UserEditor to DataEditor
// Description:     create the GUI for creating the new object and property(including password) 
//                  and editing the property
//
//=============================================================================

package webwisdom.tango.config;

import com.sun.java.swing.*;

import java.awt.*;
import java.awt.event.*;

/**This class allows creating/editing the object and its properties.
 */
public class DataEditor implements ActionListener
{
	private JDialog frame;

	private JLabel nameLabel, propertyNameLabel, propertyValueLabel, passwdLabel, confirmLabel, rnLabel, emailLabel, commLabel, roleLabel;
	private JTextField nameField, propertyNameField, propertyValueField, rnField, emailField, commField, roleField;
	private JPasswordField passwdField, confirmField;

	private String name, propertyName, propertyValue, password, pwdConfirm, realname, email, community, role;
	private boolean result;
	private int typeGUI;

    public static final int USER_ADD = 0;
    public static final int PASSWORD_ADD_EDIT = 1;
    public static final int PROPERTY_ADD_EDIT = 2;
    
	/**Constructs DataEditor object for creating a new object.
	 * @param title title for DataEditor window
	 * @param type specific object type for creating a new object
	 */
	public DataEditor( Frame owner, String title, String type, String name, char echoChar )
	{
	    typeGUI = USER_ADD;
		createGUI( owner, title, type, name, echoChar );
    }

	/**Constructs DataEditor object for adding the new property.
	 * @param title title for DataEditor window
	 * @param username specific object name
	 */
	public DataEditor( Frame owner, String title, String username )
	{
		this.name = username;
		this.propertyName = "";
		this.propertyValue = "";
		typeGUI = PROPERTY_ADD_EDIT;
		createGUI( owner, title, username );
    }

	/**Constructs DataEditor object for editing the property.
	 * @param title title for DataEditor window
	 * @param ObjName specific object name for editing its property
	 * @param propertyname specific property for editing
	 * @param propertyvalue original value of the property
	 */
	public DataEditor( Frame owner, String title, String ObjName, String propertyname, String propertyvalue )
	{
		this.name = ObjName;
		this.propertyName = propertyname;
		this.propertyValue = propertyvalue;
		typeGUI = PROPERTY_ADD_EDIT;
		createGUI( owner, title, ObjName, propertyname, propertyvalue );
	}

	/**Constructs DataEditor object for adding a password.
	 * @param title title for DataEditor window
	 * @param ObjName specific object name for adding its password
	 * @param echoChar echo character for a password
	 */
	public DataEditor( Frame owner, String title, String ObjName, char echoChar )
	{
		this.name = ObjName;
		typeGUI = PASSWORD_ADD_EDIT;
		createGUI( owner, title, ObjName, echoChar );
	}
	
	//
	// ActionListener
	//
	public void actionPerformed( ActionEvent e )
	{
		String command = e.getActionCommand();
		if( command.compareTo( "OK" ) == 0 )
		{
			result = true;
			name = nameField.getText();
			if ( typeGUI == USER_ADD )
			{
			    if ( passwdField.getText().equals( confirmField.getText() ) )
			    {
			        password = passwdField.getText();
			        pwdConfirm = confirmField.getText();
			        realname = rnField.getText();
			        email = emailField.getText();
			        community = commField.getText();
			        role = roleField.getText();
			    }
			    else
			    {
			        JOptionPane.showMessageDialog( frame, "Not matched password. ", "Password", JOptionPane.ERROR_MESSAGE );
			    }
			}
			else if ( typeGUI == PASSWORD_ADD_EDIT )
			{
			    if ( passwdField.getText().equals( confirmField.getText() ) )
			    {
			        password = passwdField.getText();
			        pwdConfirm = confirmField.getText();
			    }
			    else
			    {
			        JOptionPane.showMessageDialog( frame, "Not matched password. ", "Password", JOptionPane.ERROR_MESSAGE );
			    }
			}
			else if ( typeGUI == PROPERTY_ADD_EDIT )
			{
		        propertyName = propertyNameField.getText();
		        propertyValue = propertyValueField.getText();
			}
			
			frame.setVisible( false );
		}
		else if( command.compareTo( "Cancel" ) == 0 )
		{
			result = false;
			frame.setVisible( false );
		}
	}

	public boolean getResult()
	{
		return result;
	}

	public String getValueAt( int i )
	{
		return name;
	}

	public String getName()
	{
		return name;
	}

	public String getPropertyValue()
	{
		return propertyValue;
	}

	public String getPropertyName()
	{
		return propertyName;
	}

	public String getPassword()
	{
		return password;
	}
	
	public String getRealName()
	{
	    return realname;
	}
	
    public String getEmail()
	{
	    return email;
	}
	
	public String getCommunity()
	{
	    return community;
	}
	
	public String getRole()
	{
	    return role;
	}

	//
	// private methods
	//

	void createGUI( Frame owner, String title, String type, String name, char echoChar )
	{
	    // create GUI for adding the new object
	    
		frame = new JDialog( owner, title, true );

		nameLabel = new JLabel( "Name:" );
		rnLabel = new JLabel( "Real Name:" );
		passwdLabel = new JLabel( "Password:" );
		confirmLabel = new JLabel( "Confirm:" );		
		emailLabel = new JLabel( "E-Mail:" );
		commLabel = new JLabel( "Community:" );
		roleLabel = new JLabel( "Role:" );

		nameField = new JTextField( name, 22 );
		nameField.setText( name );
		nameField.setEditable( false );
		rnField = new JTextField( realname, 22 );
		passwdField = new JPasswordField( password, 22 );
		passwdField.setEchoChar( echoChar );
		confirmField = new JPasswordField( pwdConfirm, 22 );
		confirmField.setEchoChar( echoChar );
		emailField = new JTextField( email, 22 );
		commField = new JTextField( community, 22 );
		roleField = new JTextField( role, 22 );
		
		// layout the labels in a panel
		JPanel labelPane = new JPanel();
		labelPane.setLayout( new GridLayout( 0, 1 ) );
		labelPane.add( nameLabel );
		labelPane.add( rnLabel );
		labelPane.add( passwdLabel );
		labelPane.add( confirmLabel );
		labelPane.add( emailLabel );
		labelPane.add( commLabel );
		labelPane.add( roleLabel );

		// layout the text fields in a panel
		JPanel fieldPane = new JPanel();
		fieldPane.setLayout( new GridLayout( 0, 1 ) );
		fieldPane.add( nameField );
		fieldPane.add( rnField );
		fieldPane.add( passwdField );
		fieldPane.add( confirmField );
		fieldPane.add( emailField );
		fieldPane.add( commField );
		fieldPane.add( roleField );

		// layout the buttons in a panel
		JPanel buttonPane = new JPanel();
		buttonPane.setBorder( BorderFactory.createEmptyBorder( 5, 10, 5, 10 ) );
		buttonPane.setLayout( new GridLayout( 1, 0, 10, 5 ) );
		JLabel l = new JLabel( "" );
		buttonPane.add( l );
		JButton button = new JButton( "OK" );
		button.addActionListener( this );
		button.setVerticalTextPosition( AbstractButton.CENTER );
        button.setHorizontalTextPosition( AbstractButton.CENTER );
		buttonPane.add( button );
		button = new JButton( "Cancel" );
		button.addActionListener( this );
		button.setVerticalTextPosition( AbstractButton.CENTER );
        button.setHorizontalTextPosition( AbstractButton.CENTER );
		buttonPane.add( button );
		l = new JLabel( "" );
		buttonPane.add( l );

		// put the panels in another panel, labels on left,
		// text fields on right
		JPanel contentPane = new JPanel();
		contentPane.setBorder( BorderFactory.createEmptyBorder( 20, 20, 5, 20 ) );
		contentPane.setLayout( new BorderLayout() );
		contentPane.add( labelPane, BorderLayout.CENTER );
		contentPane.add( fieldPane, BorderLayout.EAST );
		contentPane.add( buttonPane, BorderLayout.SOUTH );

		frame.setContentPane( contentPane );
		frame.pack();
		frame.setVisible( true );
	}
    
	void createGUI( Frame owner, String title, String name )
	{
	    // create GUI for adding the new property of the given name's object
	    
		frame = new JDialog( owner, title, true );

		nameLabel = new JLabel( "Name:" );
		propertyNameLabel = new JLabel( "Property:" );
		propertyValueLabel = new JLabel( "Value:" );		

		nameField = new JTextField( name, 22 );
		nameField.setText( name );
		nameField.setEditable( false );
		propertyNameField = new JTextField( propertyName, 22 );
		propertyNameField.requestFocus();
		propertyValueField = new JTextField( propertyValue, 22 );
		
		// layout the labels in a panel
		JPanel labelPane = new JPanel();
		labelPane.setLayout( new GridLayout( 0, 1 ) );
		labelPane.add( nameLabel );
		labelPane.add( propertyNameLabel );
		labelPane.add( propertyValueLabel );

		// layout the text fields in a panel
		JPanel fieldPane = new JPanel();
		fieldPane.setLayout( new GridLayout( 0, 1 ) );
		fieldPane.add( nameField );
		fieldPane.add( propertyNameField );
		fieldPane.add( propertyValueField );

		// layout the buttons in a panel
		JPanel buttonPane = new JPanel();
		buttonPane.setBorder( BorderFactory.createEmptyBorder( 5, 10, 5, 10 ) );
		buttonPane.setLayout( new GridLayout( 1, 0, 10, 5 ) );
		JLabel l = new JLabel( "" );
		buttonPane.add( l );
		JButton button = new JButton( "OK" );
		button.addActionListener( this );
		button.setVerticalTextPosition( AbstractButton.CENTER );
        button.setHorizontalTextPosition( AbstractButton.CENTER );
		buttonPane.add( button );
		button = new JButton( "Cancel" );
		button.addActionListener( this );
		button.setVerticalTextPosition( AbstractButton.CENTER );
        button.setHorizontalTextPosition( AbstractButton.CENTER );
		buttonPane.add( button );
		l = new JLabel( "" );
		buttonPane.add( l );

		// put the panels in another panel, labels on left,
		// text fields on right
		JPanel contentPane = new JPanel();
		contentPane.setBorder( BorderFactory.createEmptyBorder( 20, 20, 5, 20 ) );
		contentPane.setLayout( new BorderLayout() );
		contentPane.add( labelPane, BorderLayout.CENTER );
		contentPane.add( fieldPane, BorderLayout.EAST );
		contentPane.add( buttonPane, BorderLayout.SOUTH );

		frame.setContentPane( contentPane );
		frame.pack();
		frame.setVisible( true );
	}

	void createGUI( Frame owner, String title, String name, String property, String value )
	{
	    // create GUI for editing the new property of the given name's user
	    
		frame = new JDialog( owner, title, true );

		nameLabel = new JLabel( "Name:" );
		propertyNameLabel = new JLabel( "Property:" );
		propertyValueLabel = new JLabel( "Value:" );		

		nameField = new JTextField( name, 22 );
		nameField.setText( name );
		nameField.setEditable( false );
		propertyNameField = new JTextField( propertyName, 22 );
		propertyNameField.setText( property );
		propertyNameField.setEditable( false );		
		propertyValueField = new JTextField( propertyValue, 22 );
		propertyValueField.setText( value );
		propertyValueField.selectAll();
		propertyValueField.requestFocus();
		
		// layout the labels in a panel
		JPanel labelPane = new JPanel();
		labelPane.setLayout( new GridLayout( 0, 1 ) );
		labelPane.add( nameLabel );
		labelPane.add( propertyNameLabel );
		labelPane.add( propertyValueLabel );

		// layout the text fields in a panel
		JPanel fieldPane = new JPanel();
		fieldPane.setLayout( new GridLayout( 0, 1 ) );
		fieldPane.add( nameField );
		fieldPane.add( propertyNameField );
		fieldPane.add( propertyValueField );

		// layout the buttons in a panel
		JPanel buttonPane = new JPanel();
		buttonPane.setBorder( BorderFactory.createEmptyBorder( 5, 10, 5, 10 ) );
		buttonPane.setLayout( new GridLayout( 1, 0, 10, 5 ) );
		JLabel l = new JLabel( "" );
		buttonPane.add( l );
		JButton button = new JButton( "OK" );
		button.addActionListener( this );
		button.setVerticalTextPosition( AbstractButton.CENTER );
        button.setHorizontalTextPosition( AbstractButton.CENTER );
		buttonPane.add( button );
		button = new JButton( "Cancel" );
		button.addActionListener( this );
		button.setVerticalTextPosition( AbstractButton.CENTER );
        button.setHorizontalTextPosition( AbstractButton.CENTER );
		buttonPane.add( button );
		l = new JLabel( "" );
		buttonPane.add( l );

		// put the panels in another panel, labels on left,
		// text fields on right
		JPanel contentPane = new JPanel();
		contentPane.setBorder( BorderFactory.createEmptyBorder( 20, 20, 5, 20 ) );
		contentPane.setLayout( new BorderLayout() );
		contentPane.add( labelPane, BorderLayout.CENTER );
		contentPane.add( fieldPane, BorderLayout.EAST );
		contentPane.add( buttonPane, BorderLayout.SOUTH );

		frame.setContentPane( contentPane );
		frame.pack();
		frame.setVisible( true );
	}	

	void createGUI( Frame owner, String title, String name, char echoChar )
	{
	    // create GUI for adding the new password of the given name's user
	    
		frame = new JDialog( owner, title, true );

		nameLabel = new JLabel( "Name:" );
		passwdLabel = new JLabel( "Password:" );
		confirmLabel = new JLabel( "Confirm:" );		

		nameField = new JTextField( name, 22 );
		nameField.setText( name );
		nameField.setEditable( false );
		passwdField = new JPasswordField( 22 );
		passwdField.setText( password );
		passwdField.setEchoChar( echoChar );
		confirmField = new JPasswordField( 22 );
		confirmField.setText( pwdConfirm );
		confirmField.setEchoChar( echoChar );
		
		// layout the labels in a panel
		JPanel labelPane = new JPanel();
		labelPane.setLayout( new GridLayout( 0, 1 ) );
		labelPane.add( nameLabel );
		labelPane.add( passwdLabel );
		labelPane.add( confirmLabel );

		// layout the text fields in a panel
		JPanel fieldPane = new JPanel();
		fieldPane.setLayout( new GridLayout( 0, 1 ) );
		fieldPane.add( nameField );
		fieldPane.add( passwdField );
		fieldPane.add( confirmField );

		// layout the buttons in a panel
		JPanel buttonPane = new JPanel();
		buttonPane.setBorder( BorderFactory.createEmptyBorder( 5, 10, 5, 10 ) );
		buttonPane.setLayout( new GridLayout( 1, 0, 10, 5 ) );
		JLabel l = new JLabel( "" );
		buttonPane.add( l );
		JButton button = new JButton( "OK" );
		button.addActionListener( this );
		button.setVerticalTextPosition( AbstractButton.CENTER );
        button.setHorizontalTextPosition( AbstractButton.CENTER );
		buttonPane.add( button );
		button = new JButton( "Cancel" );
		button.addActionListener( this );
		button.setVerticalTextPosition( AbstractButton.CENTER );
        button.setHorizontalTextPosition( AbstractButton.CENTER );
		buttonPane.add( button );
		l = new JLabel( "" );
		buttonPane.add( l );

		// put the panels in another panel, labels on left,
		// text fields on right
		JPanel contentPane = new JPanel();
		contentPane.setBorder( BorderFactory.createEmptyBorder( 20, 20, 5, 20 ) );
		contentPane.setLayout( new BorderLayout() );
		contentPane.add( labelPane, BorderLayout.CENTER );
		contentPane.add( fieldPane, BorderLayout.EAST );
		contentPane.add( buttonPane, BorderLayout.SOUTH );

		frame.setContentPane( contentPane );
		frame.pack();
		frame.setVisible( true );
	}	

}