//=============================================================================
//
// class DataManager
//
// Author:			Grzegorz Lewandowski
// Created:			12/01/98
// Original Class:  PasswordManager
//
// Modified:		02/10/99
// By:				Grzegorz Lewandowski
// Changes:			class name changed from PasswordManager to UserManager
// Original Class:  UserManager
//
// Modified:		07/20/99
// By:				Hyun Kim
// Changes:		    class name changed from UserManager to DataManager
// Description:     create an application for managing users and communities 
//                  for Tango Server
//
//=============================================================================

package webwisdom.tango.config;

import com.sun.java.swing.*;
import com.sun.java.swing.event.*;
import com.sun.java.swing.table.*;
import com.sun.java.swing.preview.*;

import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.util.*;

/**Main class for application managing users and communities for Tango server
 * @version 	0.9, 98/12/01
 */
public class DataManager implements ActionListener 
{
	private dataModule dm;
	private JFrame frame;
    private JTabbedPane tabbedPane;
    private JPanel panel;
	
	private JTable TableOfUserObject, TableOfUserProperties, TableOfCommObject, TableOfCommProperties;
	private JTextField valueField, CommValueField;
	private DefaultCellEditor UserPropertyValueEditor, CommPropertyValueEditor;
	
	private	JMenuBar menuBar;
	private JMenu menu, menu2, menu3, menu4;
	private	JMenuItem menuItem, fileMenuItem, dbMenuItem, httpMenuItem, ctMenuItem;
	private JMenuItem saveMenuItem, saveAsMenuItem, closeMenuItem, toFileMenuItem, toDBMenuItem;
	private JSeparator separator;
	private JPopupMenu popup;
	
	private File lastDir;
	private boolean changesMade, updated = false, closed = false;
	private String myID, myPWD, myURL, tabTitle;

    private Object[][] ListOfObjects = new Object[ 0 ][ 0 ];
  	private Object[][] ListOfProperties = new Object[ 0 ][ 0 ];

	/**Constructs new DataManager object
	 */
	public DataManager() throws IOException
	{
		// initialize log
		// System.out.print( "initializing log..." );
		//webwisdom.tango.test.Log.init( 10 );
		// System.out.println( " DONE" );
		
		// create dataModule object
        dm = new dataModule();

		// create GUI
        frame = new JFrame( "Tango Server Data Manager" );

		//  menus

		// System.out.println( " creating menus..." );
		menuBar = new JMenuBar();
		frame.setJMenuBar( menuBar );

		menu = new JMenu( "Protocol" );
		menuBar.add( menu );
		
		fileMenuItem = new JMenuItem( "Open File..." );
		fileMenuItem.setActionCommand( "openFile" );
		fileMenuItem.addActionListener( this );
		menu.add( fileMenuItem );
		
		dbMenuItem = new JMenuItem( "Connect Database" );
		dbMenuItem.setActionCommand( "connectDatabase" );
		dbMenuItem.addActionListener( this );
		menu.add( dbMenuItem );	
		
		httpMenuItem = new JMenuItem( "Establish HTTP Connection" );
		httpMenuItem.setActionCommand( "establishHTTP" );
		httpMenuItem.addActionListener( this );
		menu.add( httpMenuItem );		
		
		separator  = new JSeparator();
		menu.add( separator );
		
		saveMenuItem = new JMenuItem( "Save and Close" );
		saveMenuItem.setActionCommand( "SaveAndClose" );
		saveMenuItem.addActionListener( this );
		saveMenuItem.setEnabled( false );
		menu.add( saveMenuItem );
		
		saveAsMenuItem = new JMenuItem( "Save As... and Close" );
		saveAsMenuItem.setActionCommand( "saveAsFile" );
		saveAsMenuItem.addActionListener( this );
		saveAsMenuItem.setEnabled( false );
		menu.add( saveAsMenuItem );
				
		closeMenuItem = new JMenuItem( "Close" );
		closeMenuItem.setActionCommand( "close" );
		closeMenuItem.addActionListener( this );
		closeMenuItem.setEnabled(false);
		menu.add( closeMenuItem );		

		separator  = new JSeparator();
		menu.add( separator );
				
		menuItem = new JMenuItem( "Exit" );
		menuItem.setActionCommand( "exit" );
		menuItem.addActionListener( this );
		menu.add( menuItem );

		menu4 = new JMenu( "Tool" );
		menuBar.add( menu4 );
		menu4.setEnabled( false );
		
		toFileMenuItem = new JMenuItem( "Convert to File" );
		toFileMenuItem.setActionCommand( "convertToFile" );
		toFileMenuItem.addActionListener( this );
		menu4.add( toFileMenuItem );
		
		toDBMenuItem = new JMenuItem( "Convert to Database" );
		toDBMenuItem.setActionCommand( "convertToDatabase" );
		toDBMenuItem.addActionListener( this );
		menu4.add( toDBMenuItem );
		
		separator  = new JSeparator();
		menu4.add( separator );
		
        ctMenuItem = new JMenuItem( "Create Tables..." );
		ctMenuItem.setActionCommand( "createTables" );
		ctMenuItem.addActionListener( this );
        menu4.add( ctMenuItem );
        
		menu2 = new JMenu( "Object" );
		menuBar.add( menu2 );
		menu2.setEnabled( false );

		menuItem = new JMenuItem( "New Object..." );
		menuItem.setActionCommand( "newObject" );
		menuItem.addActionListener( this );
		menu2.add( menuItem );
		
		menuItem = new JMenuItem( "Delete Object" );
		menuItem.setActionCommand( "deleteObject" );
		menuItem.addActionListener( this );
		menu2.add( menuItem );

		menuItem = new JMenuItem( "All List..." );
		menuItem.setActionCommand( "allListOfObjects" );
		menuItem.addActionListener( this );
		menu2.add( menuItem );
		
		menu3 = new JMenu( "Properties" );
		menuBar.add( menu3 );
		menu3.setEnabled( false );
		
		menuItem = new JMenuItem( "Add New Password" );
		menuItem.setActionCommand( "addPassword" );
		menuItem.addActionListener( this );
		menu3.add( menuItem );

		separator  = new JSeparator();
		menu3.add( separator );

		menuItem = new JMenuItem( "New Property..." );
		menuItem.setActionCommand( "newProperty" );
		menuItem.addActionListener( this );
		menu3.add( menuItem );

		menuItem = new JMenuItem( "Edit Property" );
		menuItem.setActionCommand( "editProperty" );
		menuItem.addActionListener( this );
		menu3.add( menuItem );
		
		menuItem = new JMenuItem( "Delete Property" );
		menuItem.setActionCommand( "deleteProperty" );
		menuItem.addActionListener( this );
		menu3.add( menuItem );

		menuItem = new JMenuItem( "All Properties..." );
		menuItem.setActionCommand( "allProperties" );
		menuItem.addActionListener( this );
		menu3.add( menuItem );
		
        //Create the popup menu.
        popup = new JPopupMenu();
        menuItem = new JMenuItem("New Object...");
        menuItem.setActionCommand( "newObject" );
        menuItem.addActionListener(this);
        popup.add(menuItem);
        menuItem = new JMenuItem("Delete Object");
        menuItem.setActionCommand( "deleteObject" );
        menuItem.addActionListener(this);
        popup.add(menuItem);
        menuItem = new JMenuItem( "Show All List..." );
		menuItem.setActionCommand( "allListOfObjects" );
		menuItem.addActionListener( this );
		popup.add( menuItem );
        separator  = new JSeparator();
		popup.add( separator );
        menuItem = new JMenuItem( "New Property..." );
		menuItem.setActionCommand( "newProperty" );
		menuItem.addActionListener( this );
		popup.add( menuItem );
		menuItem = new JMenuItem( "Edit Property" );
		menuItem.setActionCommand( "editProperty" );
		menuItem.addActionListener( this );
		popup.add( menuItem );
		menuItem = new JMenuItem( "Delete Property" );
		menuItem.setActionCommand( "deleteProperty" );
		menuItem.addActionListener( this );
		popup.add( menuItem );
		separator  = new JSeparator();
		popup.add( separator );
		menuItem = new JMenuItem( "Add New Password" );
		menuItem.setActionCommand( "addPassword" );
		menuItem.addActionListener( this );
		popup.add( menuItem );
		 		              
		// System.out.println( " adding listeners..." );
		frame.addWindowListener( new WindowAdapter()
		{
            public void windowClosing( WindowEvent e )
			{
				try
				{
					if( closeUserInfoFile() )
					{
				        if ( !changesMade )
				        {
				            // just close w/o saving
				            dm.close();
				        }
				        fileMenuItem.setEnabled(true);
			            dbMenuItem.setEnabled(true);
				        httpMenuItem.setEnabled(true);
				        saveMenuItem.setEnabled(false);
				        saveAsMenuItem.setEnabled(false);
			            closeMenuItem.setEnabled(false);
			            menu2.setEnabled(false);
		                menu3.setEnabled(false);
						System.exit( 0 );
					}
				}
				catch( IOException ex )
				{
				}
            }
        } );

		
		// add the panel to the tabbed panel
	    tabbedPane = new JTabbedPane();
        tabbedPane.addTab( "Object Type", createTableContent( "user" ) );
        tabbedPane.setSelectedIndex( -1 );
        tabbedPane.setEnabledAt ( 0, false );
        
	    tabbedPane.addChangeListener( new ChangeListener ()
	    {
	        public void stateChanged( ChangeEvent e )
	        {
	            int i = tabbedPane.getSelectedIndex();
	            if ( i >= 0 )
	            {
	                tabTitle = tabbedPane.getTitleAt( i );
	                showAllObjects();
	            }
	        }
	    } );
	    
        frame.getContentPane().add(tabbedPane, BorderLayout.WEST);
	    
		// init last directory store
		lastDir = null;
		
		changesMade = false;
		
        frame.pack();
		frame.setVisible( true );
		// System.out.println( "GUI ready" );
	}
	    
	//
	// main
	//
	public static void main( String[] args )
	{
        try
		{
            UIManager.setLookAndFeel( UIManager.getCrossPlatformLookAndFeelClassName() );
			// UIManager.setLookAndFeel( UIManager.getSystemLookAndFeelClassName() );
        }
		catch( Exception e )
		{
			System.err.println( "WHAT?!!!" );
			System.err.println( "FATAL ERROR: " + e.getMessage() );
			return;
		}

		// create itself
		DataManager dm = null;
		try
		{
			dm = new DataManager();
		}
		catch( IOException e )
		{
			System.err.println( "Unable to create DataManager. " + e.getMessage() );
		}
	}

	//
	// ActionListener
	//
	public void actionPerformed( ActionEvent e )
	{
		String command = e.getActionCommand();
		if( command.compareTo( "openFile" ) == 0 )
		{
			try
			{
				if( !closeUserInfoFile() )
				{
					return;
				}
			}
			catch( IOException ex )
			{
				System.err.println( "Unable to save the data. " + ex.getMessage() );
				JOptionPane.showMessageDialog( frame, "Unable to save the data. " + ex.getMessage(), "Tango User Manager", JOptionPane.ERROR_MESSAGE );
				return;
			}
			
			JFileChooser chooser = new JFileChooser();
			chooser.setFileSelectionMode( JFileChooser.FILES_ONLY );
			ExtensionFileFilter filter = new ExtensionFileFilter( ".dat", "Tango Server Data Files (*.dat)" );
			chooser.addChoosableFileFilter( filter );
			chooser.setFileFilter( filter );
			if( lastDir != null )
			{
				chooser.setCurrentDirectory( lastDir );
			}
			
			int result = chooser.showOpenDialog( frame );
			if( result == JFileChooser.APPROVE_OPTION )
			{
				// store current directory for future use
				lastDir = chooser.getCurrentDirectory();
				try
				{
					openFile( chooser.getSelectedFile().getPath() );
				}
				catch( Exception ex )
				{
					// System.err.println( "Unable to open file. " + ex.getMessage() );
					JOptionPane.showMessageDialog( frame,
												   "Unable to open file. " + ex.getMessage(),
												   "Open File",
												   JOptionPane.ERROR_MESSAGE );
				}
			}			
		}
		else if( command.compareTo( "connectDatabase" ) == 0 )
		{
			try
			{
				if( !closeUserInfoFile() )
				{
					return;
				}
			}
			catch( IOException ex )
			{
				System.err.println( "Unable to save the data. " + ex.getMessage() );
				JOptionPane.showMessageDialog( frame, "Unable to save the data. " + ex.getMessage(), "Tango User Manager", JOptionPane.ERROR_MESSAGE );
				return;
			}
			
			try
			{
				connectDatabase();
			}
			catch( Exception ex )
			{
				// System.err.println( "Unable to connect to database. " + ex.getMessage() );
				JOptionPane.showMessageDialog( frame,
											   "Unable to connect to database. " + ex.getMessage(),
											   "Connect Database",
											   JOptionPane.ERROR_MESSAGE );
			}
		}
		else if( command.compareTo( "establishHTTP" ) == 0 )
		{
			try
			{
				if( !closeUserInfoFile() )
				{
					return;
				}
			}
			catch( IOException ex )
			{
				//System.err.println( "Unable to save the data. " + ex.getMessage() );
				JOptionPane.showMessageDialog( frame, "Unable to save the data. " + ex.getMessage(), "Tango User Manager", JOptionPane.ERROR_MESSAGE );
				return;
			}
			
			try
			{
				establishHTTP();
			}
			catch( Exception ex )
			{
				// System.err.println( "Unable to establish http connection. " + ex.getMessage() );
				JOptionPane.showMessageDialog( frame,
											   "Unable to establish HTTP connection. " + ex.getMessage(),
											   "Establish HTTP Connection",
											   JOptionPane.ERROR_MESSAGE );
			}
		}
		else if( command.compareTo( "SaveAndClose" ) == 0 )
		{			
			fileMenuItem.setEnabled(true);
			dbMenuItem.setEnabled(true);
			httpMenuItem.setEnabled(true);
			saveMenuItem.setEnabled(false);
			saveAsMenuItem.setEnabled(false);
			closeMenuItem.setEnabled(false);
			menu2.setEnabled(false);
			menu3.setEnabled(false);
			menu4.setEnabled(false);
			
			ListOfObjects = new Object[ 0 ][ 0 ];
			ListOfProperties = new Object[ 0 ][ 0 ];
    		ObjTableModel dtm = (ObjTableModel)TableOfUserObject.getModel();
	    	dtm.recreate( ListOfObjects );
			PropertyTableModel ptm = (PropertyTableModel)TableOfUserProperties.getModel();
	    	ptm.recreate( ListOfProperties );
			
            while ( tabbedPane.getTabCount() > 0 )
	        {
	            tabbedPane.setEnabledAt( 0, false );
	            tabbedPane.removeTabAt( 0 );
	        }
			frame.repaint();
			
			dm.update();
			changesMade = false;			
		}
		else if( command.compareTo( "saveAsFile" ) == 0 )
		{
			JFileChooser chooser = new JFileChooser();
			chooser.setFileSelectionMode( JFileChooser.FILES_ONLY );
			ExtensionFileFilter filter = new ExtensionFileFilter( ".dat", "Tango Server Data Files (*.dat)" );
			chooser.addChoosableFileFilter( filter );
			chooser.setFileFilter( filter );
			if( lastDir != null )
			{
				chooser.setCurrentDirectory( lastDir );
			}
			
			int result = chooser.showSaveDialog( frame );
			if( result == JFileChooser.APPROVE_OPTION )
			{
				// store current directory for future use
				lastDir = chooser.getCurrentDirectory();
						        
		        fileMenuItem.setEnabled(true);
		        dbMenuItem.setEnabled(true);
		        httpMenuItem.setEnabled(true);
		        saveMenuItem.setEnabled(false);
		        saveAsMenuItem.setEnabled(false);
		        closeMenuItem.setEnabled(false);
		        menu2.setEnabled(false);
			    menu3.setEnabled(false);
			    menu4.setEnabled(false);
			
		        ListOfObjects = new Object[ 0 ][ 0 ];
		        ListOfProperties = new Object[ 0 ][ 0 ];
		        ObjTableModel otm = (ObjTableModel)TableOfUserObject.getModel();
    	        otm.recreate( ListOfObjects );
		        PropertyTableModel ptm = (PropertyTableModel)TableOfUserProperties.getModel();
    	        ptm.recreate( ListOfProperties );

                while ( tabbedPane.getTabCount() > 0 )
                {
	                tabbedPane.setEnabledAt( 0, false );
	                tabbedPane.removeTabAt( 0 );
	            }
                frame.repaint();

		        dm.update( chooser.getSelectedFile().getPath() );
		        changesMade = false;
			}
		}		
		else if( command.compareTo( "close" ) == 0 )
		{
			try
			{
				if ( closeUserInfoFile() )
				{
    				fileMenuItem.setEnabled(true);
	    		    dbMenuItem.setEnabled(true);
		    		httpMenuItem.setEnabled(true);
			    	saveMenuItem.setEnabled(false);
				    saveAsMenuItem.setEnabled(false);
    			    closeMenuItem.setEnabled(false);
	    		    menu2.setEnabled(false);
		            menu3.setEnabled(false);
		            menu4.setEnabled(false);
			        
    			    ListOfObjects = new Object[ 0 ][ 0 ];
	    		    ListOfProperties = new Object[ 0 ][ 0 ];
        		    ObjTableModel otm = (ObjTableModel)TableOfUserObject.getModel();
	            	otm.recreate( ListOfObjects );
		        	PropertyTableModel ptm = (PropertyTableModel)TableOfUserProperties.getModel();
        	    	ptm.recreate( ListOfProperties );

        	    	while ( tabbedPane.getTabCount() > 0 )
	                {
	                    tabbedPane.setEnabledAt( 0, false );
	                    tabbedPane.removeTabAt( 0 );
	                }
                    frame.repaint();
				    
				    if ( updated )
				    {
				        dm.update();
				        updated = false;
				    }
				    else
				    {
				        // just close w/o saving
				        dm.close();
				    }
			        changesMade = false;
			    }
			}
			catch( IOException ex )
			{
				System.out.println( "Unable to save data. " + ex.getMessage() );
				JOptionPane.showMessageDialog( frame, "Unable to save the data. " + ex.getMessage(), "Exit", JOptionPane.ERROR_MESSAGE );
			}		    
		}
		else if( command.compareTo( "exit" ) == 0 )
		{
			try
			{
				if( closeUserInfoFile() )
				{
    				fileMenuItem.setEnabled(true);
	    		    dbMenuItem.setEnabled(true);
		    		httpMenuItem.setEnabled(true);
			    	saveMenuItem.setEnabled(false);
				    saveAsMenuItem.setEnabled(false);
    			    closeMenuItem.setEnabled(false);
	    		    menu2.setEnabled(false);
		            menu3.setEnabled(false);
		            menu4.setEnabled(false);
		            
				    if ( updated )
				    {
				        dm.update();
				        updated = false;
				    }
				    else if ( closed )
				    {
				        // just close w/o saving
				        dm.close();
				        closed = false;
				    }
			        changesMade = false;
				    
					System.exit( 0 );
				}
			}
			catch( IOException ex )
			{
				System.out.println( "Unable to save data. " + ex.getMessage() );
				JOptionPane.showMessageDialog( frame, "Unable to save data. " + ex.getMessage(), "Exit", JOptionPane.ERROR_MESSAGE );
			}
		}
		else if( command.compareTo( "convertToFile" ) == 0 )
		{
			JFileChooser chooser = new JFileChooser();
			chooser.setFileSelectionMode( JFileChooser.FILES_ONLY );
			ExtensionFileFilter filter = new ExtensionFileFilter( ".dat", "Tango Server Data Files (*.dat)" );
			chooser.addChoosableFileFilter( filter );
			chooser.setFileFilter( filter );
			if( lastDir != null )
			{
				chooser.setCurrentDirectory( lastDir );
			}
			
			int result = chooser.showSaveDialog( frame );
			if( result == JFileChooser.APPROVE_OPTION )
			{
				// store current directory for future use
				lastDir = chooser.getCurrentDirectory();
				
				dataModule newdm = new dataModule();
				try
				{
	                newdm.open( "file", chooser.getSelectedFile().getPath(), null, null);
	            }
	            catch ( Exception ex )
	            {
	                JOptionPane.showMessageDialog( frame, "Unable to open file. ", "Format Conversion", JOptionPane.ERROR_MESSAGE );
	            }
	            
	            // retrieve all data from the currently open protocol and store them into file
	            try
	            {
	                for ( int i = 0; i < tabbedPane.getTabCount(); i++ )
	                {
	                    String type = tabbedPane.getTitleAt( i );
	                    Enumeration objects = dm.getObjects( type );
	                    while ( objects.hasMoreElements() )
	                    {
	                        String ObjName = objects.nextElement().toString();
	                        newdm.addObject( type, ObjName );
	                        Enumeration keys = dm.getObjectKeys( type, ObjName );
	                        while ( keys.hasMoreElements() )
	                        {
	                            String keyname = keys.nextElement().toString();
	                            newdm.setValue( type, ObjName, keyname, dm.getValue( type, ObjName, keyname) );
	                        }
	                    }
	                }
	                newdm.update();
				    JOptionPane.showMessageDialog( frame, "Completed to convert data to file. ", "Format Conversion", JOptionPane.INFORMATION_MESSAGE );
	            }
	            catch ( Exception ex )
	            {
	                JOptionPane.showMessageDialog( frame, "Unable to convert data to file. ", "Format Conversion", JOptionPane.ERROR_MESSAGE );
	            }
			}
			else
			{
			    JOptionPane.showMessageDialog( frame, "Unable to convert data to file. ", "Format Conversion", JOptionPane.ERROR_MESSAGE );
			}
		}
		else if( command.compareTo( "convertToDatabase" ) == 0 )
		{
		    ProtocolBox pb = new ProtocolBox( frame, "LoginID/Password" );
		    if ( pb.getResult() )
		    {
			    dataModule newdm = new dataModule();
			    try 
			    {
                    newdm.open( "jdbc", pb.getURL(), pb.getID(), pb.getPassword() );
	            
                    // retrieve all data from the currently open protocol and store them into database
                    for ( int i = 0; i < tabbedPane.getTabCount(); i++ )
                    {
                        String type = tabbedPane.getTitleAt( i );
                        Enumeration objects = dm.getObjects( type );
                        while ( objects.hasMoreElements() )
                        {
                            String ObjName = objects.nextElement().toString();
                            newdm.addObject( type, ObjName );
                            Enumeration keys = dm.getObjectKeys( type, ObjName );
                            while ( keys.hasMoreElements() )
                            {
                                String keyname = keys.nextElement().toString();
                                newdm.setValue( type, ObjName, keyname, dm.getValue( type, ObjName, keyname) );
                            }
                        }
                    }
                    newdm.update();
                    JOptionPane.showMessageDialog( frame, "Completed to convert data to database. ", "Format Conversion", JOptionPane.INFORMATION_MESSAGE );
                }
                catch ( Exception ex )
                {
                    JOptionPane.showMessageDialog( frame, "Unable to convert data to database. " + ex.getMessage(), "Format Conversion", JOptionPane.ERROR_MESSAGE );
                }
	        }
    	}
    	else if( command.compareTo( "createTables" ) == 0 )
    	{
    	    createTables ct = new createTables();
    	    try
    	    {
    	        ct.open( myID, myPWD, myURL );
    	        ct.create();
    	        ct.close();
    	        
   	            JOptionPane.showMessageDialog( frame, "Completed to create tables. ", "Table Creation", JOptionPane.INFORMATION_MESSAGE );
    	    }
    	    catch ( Exception ex )
    	    {
    	        JOptionPane.showMessageDialog( frame, "Unable to create tables. " + ex.getMessage(), "Table Creation", JOptionPane.ERROR_MESSAGE );
    	    }
    	}
		else if( command.compareTo( "newObject" ) == 0 )
		{
		    String ObjName = JOptionPane.showInputDialog( frame, "Enter the new name. ", "Create New Object", JOptionPane.INFORMATION_MESSAGE );
			if ( ObjName != null )
			{
				// modify data		
			    try
			    {
			        addNewObject( ObjName );
			    }
			    catch( Exception ex )
			    {
				    System.out.println( "Unable to modify the data. " + ex.getMessage() );
				    JOptionPane.showMessageDialog( frame, "Unable to modify the data. " + ex.getMessage(), "Create New Object", JOptionPane.ERROR_MESSAGE );
			    }
			}
		}
		else if( command.compareTo( "deleteObject" ) == 0 )
		{
			int result = JOptionPane.showConfirmDialog( frame,
														"Do you want to delete this?",
														"Tango Data Manager",
														JOptionPane.YES_NO_OPTION );
			if( result == JOptionPane.YES_OPTION )
			{
				// delete
				int n = -1;
				if ( tabTitle.equals( "user" ) )
				{
			        n = TableOfUserObject.getSelectedRow();
			    }
			    else if ( tabTitle.equals( "community" ) )
			    {
			        n = TableOfCommObject.getSelectedRow();
			    }
    			try
	    		{
			    	deleteObject( n );
			    }
    			catch( Exception ex )
	    		{
		    		// System.err.println( "Unable to modify user data. " + ex.getMessage() );
			    	JOptionPane.showMessageDialog( frame, "Unable to modify the data. " + ex.getMessage(), "Delete Object", JOptionPane.ERROR_MESSAGE );
			    }
			}		    
		}
		else if( command.compareTo( "allListOfObjects" ) == 0 )
		{
			showAllObjects ();
		}		
		else if( command.compareTo( "newProperty" ) == 0 )
		{
		    int n = -1;
		    if ( tabTitle.equals( "user" ) )
		    {
			    n = TableOfUserObject.getSelectedRow();
			}
			else if ( tabTitle.equals( "community" ) )
			{
			    n = TableOfCommObject.getSelectedRow();
			}
		    
			try
			{
				addNewProperty( n );
			}
			catch( Exception ex )
			{
				// System.err.println( "Unable to modify user data. " + ex.getMessage() );
				JOptionPane.showMessageDialog( frame, "Unable to modify the data. " + ex.getMessage(), "Delete Object", JOptionPane.ERROR_MESSAGE );
			}
		}
		else if( command.compareTo( "editProperty" ) == 0 )
		{
		    int selectedObject = -1;
		    int selectedProperty = -1;
		    if ( tabTitle.equals( "user" ) )
		    {
			    selectedObject = TableOfUserObject.getSelectedRow();
			    selectedProperty = TableOfUserProperties.getSelectedRow();
			}
			else if ( tabTitle.equals( "community" ) )
			{
			    selectedObject = TableOfCommObject.getSelectedRow();
			    selectedProperty = TableOfCommProperties.getSelectedRow();
			}
			try
			{
				editProperty( selectedObject, selectedProperty, "GUI" );
			}
			catch( Exception ex )
			{
				// System.err.println( "Unable to modify user data. " + ex.getMessage() );
				JOptionPane.showMessageDialog( frame, "Unable to modify the data. " + ex.getMessage(), "Delete Object", JOptionPane.ERROR_MESSAGE );
			}
		}		
		else if( command.compareTo( "deleteProperty" ) == 0 )
		{
			// delete
		    int selectedObject = -1;
		    int selectedProperty = -1;
		    if ( tabTitle.equals( "user" ) )
		    {
			    selectedObject = TableOfUserObject.getSelectedRow();
			    selectedProperty = TableOfUserProperties.getSelectedRow();
			}
			else if ( tabTitle.equals( "community" ) )
			{
			    selectedObject = TableOfCommObject.getSelectedRow();
			    selectedProperty = TableOfCommProperties.getSelectedRow();
			}
		    try
		    {
   				deleteProperty( selectedObject, selectedProperty );
    		}
	    	catch( Exception ex )
		    {
			    // System.err.println( "Unable to modify user data. " + ex.getMessage() );
   				JOptionPane.showMessageDialog( frame, "Unable to modify the data. " + ex.getMessage(), "Delete Object", JOptionPane.ERROR_MESSAGE );
    		}
		}
		else if( command.compareTo( "allProperties" ) == 0 )
		{
		    ObjTableModel otm = null;
		    int row = -1;
		    if ( tabTitle.equals( "user" ) )
		    {
			    otm = (ObjTableModel)TableOfUserObject.getModel();
			    row = TableOfUserObject.getSelectedRow();
			}
			else if ( tabTitle.equals( "community" ) )
			{
			    otm = (ObjTableModel)TableOfCommObject.getModel();
			    row = TableOfCommObject.getSelectedRow();
			}
			showProperties( (String)otm.getValueAt( row, 0 ) );
		}		
		else if( command.compareTo( "addPassword" ) == 0 )
		{
		    int n = -1;
		    if ( tabTitle.equals( "user" ) )
		    {
			    n = TableOfUserObject.getSelectedRow();
			}
			else if ( tabTitle.equals( "community" ) )
			{
			    n = TableOfCommObject.getSelectedRow();
			}
			try
			{
			    addPassword( n );
			}
			catch( Exception ex )
			{
			    // System.err.println( "Unable to modify user data. " + ex.getMessage() );
			    JOptionPane.showMessageDialog( frame, "Unable to modify the data. " + ex.getMessage(), "Delete Object", JOptionPane.ERROR_MESSAGE );
			}
		}		
	}


	//
	// private methods
	//
	private void openFile( String filename ) throws Exception
	{
		dm.open( "file", filename, null, null );
	    
		// table of objects
		// System.out.println( " creating table..." );
		createTabbedPane();

		frame.setTitle( "Tango Server Data Manager - " + filename );
		changesMade = false;
		frame.pack();
		frame.repaint();		    
		
		fileMenuItem.setEnabled(false);
		dbMenuItem.setEnabled(false);
		httpMenuItem.setEnabled(false);
		saveMenuItem.setEnabled(true);
		saveAsMenuItem.setEnabled(true);
		menu2.setEnabled(true);
		menu3.setEnabled(true);
		menu4.setEnabled(true);
		closeMenuItem.setEnabled(true);
		toFileMenuItem.setEnabled(false);
		toDBMenuItem.setEnabled(true);
		ctMenuItem.setEnabled(false);
	}

	private void connectDatabase() throws Exception
	{
	    ProtocolBox pb = new ProtocolBox( frame, "Login ID/Password" );
	    if ( pb.getResult() )
	    {
	        myID = pb.getID();
	        myPWD = pb.getPassword();
	        myURL = pb.getURL();
		    dm.open( "jdbc", myURL, myID, myPWD );

		    // table of objects
		    // System.out.println( " creating table..." );
			createTabbedPane();
					
		    frame.setTitle( "Tango Server Data Manager - " + myURL );
		    changesMade = false;
		    frame.pack();
		    frame.repaint();
		    
			fileMenuItem.setEnabled(false);
			dbMenuItem.setEnabled(false);
			httpMenuItem.setEnabled(false);
			saveMenuItem.setEnabled(true);
			saveAsMenuItem.setEnabled(false);
			closeMenuItem.setEnabled(true);
			menu2.setEnabled(true);
		    menu3.setEnabled(true);
		    menu4.setEnabled(true);
		    toFileMenuItem.setEnabled(true);
		    toDBMenuItem.setEnabled(false);
		    ctMenuItem.setEnabled(true);
		}
	}
	
	private void establishHTTP() throws Exception
	{
	    ProtocolBox pb = new ProtocolBox( frame, "Establish HTTP Connection", "url" );
	    if ( pb.getResult() )
	    {
		    dm.open( "web", pb.getURL(), null, null );

		    // table of objects
		    // System.out.println( " creating table..." );
			createTabbedPane();
								
		    frame.setTitle( "Tango Server Data Manager - " + pb.getURL() );
		    changesMade = false;
		    frame.pack();
		    frame.repaint();
		    
			fileMenuItem.setEnabled(false);
			dbMenuItem.setEnabled(false);
			httpMenuItem.setEnabled(false);
			saveMenuItem.setEnabled(false);
			saveAsMenuItem.setEnabled(false);
			closeMenuItem.setEnabled(true);
			menu2.setEnabled(true);
		    menu3.setEnabled(true);
		    menu4.setEnabled(true);
		    toFileMenuItem.setEnabled(true);
		    toDBMenuItem.setEnabled(true);
		    ctMenuItem.setEnabled(false);
		}
	}
	
	private void createTabbedPane()
	{  	
        while ( tabbedPane.getTabCount() > 0 )
	    {
	        tabbedPane.setEnabledAt( 0, false );
	        tabbedPane.removeTabAt( 0 );
	    }
	    
        tabbedPane.addTab( "user", createTableContent( "user" ) );
        tabbedPane.addTab( "community", createTableContent( "community" ) );
        showAllObjects();
        
        if ( tabbedPane.getTabCount() > 0 )
        {
            tabTitle = tabbedPane.getTitleAt( 0 );
            tabbedPane.setSelectedIndex( 0 );
        }

        //Add tabbedPane to this frame.
        frame.getContentPane().add(tabbedPane, BorderLayout.WEST);
		frame.pack();
		frame.repaint();
    }

	private JPanel createTableContent( String type )
	{
	    if ( type.equals( "user" ) )
	    {
    	    int i = 0;
		    TableModel model = new ObjTableModel( ListOfObjects );
		    TableOfUserObject = new JTable( model );
		    TableOfUserObject.setPreferredScrollableViewportSize( new Dimension( 100, Math.min( 20 * i, 600 ) ) );
		    TableOfUserObject.setSelectionMode( ListSelectionModel.SINGLE_SELECTION );
            TableOfUserObject.setShowHorizontalLines( false );
		    TableOfUserObject.setShowVerticalLines( false );
		    TableOfUserObject.addMouseListener( new MouseAdapter()
		    {
			    public void mouseClicked( MouseEvent e )
			    {
				    if( e.isMetaDown() )
				    {
					    int row = TableOfUserObject.rowAtPoint( e.getPoint() );
					    //System.out.println( "right click at row " + row );
                           popup.show(e.getComponent(), e.getX(), e.getY());
				    }
				    if( e.getClickCount() == 1 )
				    {
    					int row = TableOfUserObject.rowAtPoint( e.getPoint() );
	    				ObjTableModel otm = ( ObjTableModel ) TableOfUserObject.getModel();
		    			showProperties( ( String ) otm.getValueAt( row, 0 ) );
			    	}
			    }
		    } );
		    TableOfUserObject.registerKeyboardAction( (ActionListener)this, "deleteObject", KeyStroke.getKeyStroke( java.awt.event.KeyEvent.VK_DELETE, 0 ), JComponent.WHEN_FOCUSED );
	    	TableOfUserObject.registerKeyboardAction( (ActionListener)this, "allProperties", KeyStroke.getKeyStroke( java.awt.event.KeyEvent.VK_ENTER, 0 ), JComponent.WHEN_FOCUSED );
    
    	    i = 0;
		    model = new PropertyTableModel( ListOfProperties );
		    TableOfUserProperties = new JTable( model );
/*		
		    TableColumn tc = null;
		    for ( int j = 0; j < 2; i++ ) 
		    {
                  tc = TableOfUserProperties.getColumnModel().getColumn(j);
                if (j == 1) {
                      tc.setMinWidth(200); //sport column is bigger
                } else {
                      tc.setMinWidth(100);
                }
            }
  */      
		    TableOfUserProperties.setPreferredScrollableViewportSize( new Dimension( 360, Math.min( 20 * i, 600 ) ) );
		    TableOfUserProperties.setSelectionMode( ListSelectionModel.SINGLE_SELECTION );
    		TableOfUserProperties.addMouseListener( new MouseAdapter()
    		{
	    		public void mouseClicked( MouseEvent e )
		    	{
			    	if( e.isMetaDown() )
				    {
					    int row = TableOfUserProperties.rowAtPoint( e.getPoint() );
					    //System.out.println( "right click at row " + row );
					    popup.show(e.getComponent(), e.getX(), e.getY());
				    }
			    }
		    } );
		    TableOfUserProperties.registerKeyboardAction( (ActionListener)this, "editProperty", KeyStroke.getKeyStroke( java.awt.event.KeyEvent.VK_ENTER, 0 ), JComponent.WHEN_FOCUSED );
		    //TableOfUserProperties.registerKeyboardAction( (ActionListener)this, "deleteProperty", KeyStroke.getKeyStroke( java.awt.event.KeyEvent.VK_DELETE, 0 ), JComponent.WHEN_FOCUSED );

    	    valueField = new JTextField();
            valueField.setHorizontalAlignment(JTextField.LEFT);
            UserPropertyValueEditor = new DefaultCellEditor(valueField);
            TableOfUserProperties.setDefaultEditor(String.class, UserPropertyValueEditor);
            UserPropertyValueEditor.setClickCountToStart( 2 ); 		
            UserPropertyValueEditor.addCellEditorListener( new CellEditorListener()
            {
               public void editingStopped(ChangeEvent e) 
                {
    			    int selectedObj = TableOfUserObject.getSelectedRow();
			        int selectedProperty = TableOfUserProperties.getSelectedRow();
						
				    try
				    {
					    editProperty( selectedObj, selectedProperty, "CELL" );
				    }
				    catch( Exception ie )
				    {
    					JOptionPane.showMessageDialog( frame, "Unable to modify the data. " + ie.getMessage(), "Properties", JOptionPane.ERROR_MESSAGE );
	    			}
                }
            
                public void editingCanceled(ChangeEvent e)
                {
                }
            } );
        
		    // System.out.println( " creating scroller..." );
		    // create the scroll pane and add the table to it
		    JScrollPane ObjectsScrollPane = new JScrollPane( TableOfUserObject );
		    JScrollPane PropertiesScrollPane = new JScrollPane( TableOfUserProperties );
		    
		    // add the scroll pane to the panel
		    panel = new JPanel();
		    panel.setLayout( new BoxLayout ( panel, BoxLayout.X_AXIS ) );
		    panel.add( ObjectsScrollPane, BorderLayout.WEST );
		    panel.add( PropertiesScrollPane, BorderLayout.CENTER );	        		    
	    }
	    else if ( type.equals( "community" ) )
	    {
            int i = 0;
		    TableModel model = new ObjTableModel( ListOfObjects );
		    TableOfCommObject = new JTable( model );
		    TableOfCommObject.setPreferredScrollableViewportSize( new Dimension( 100, Math.min( 20 * i, 600 ) ) );
		    TableOfCommObject.setSelectionMode( ListSelectionModel.SINGLE_SELECTION );
            TableOfCommObject.setShowHorizontalLines( false );
		    TableOfCommObject.setShowVerticalLines( false );
		    TableOfCommObject.addMouseListener( new MouseAdapter()
			{
				public void mouseClicked( MouseEvent e )
				{
					if( e.isMetaDown() )
					{
						int row = TableOfCommObject.rowAtPoint( e.getPoint() );
						//System.out.println( "right click at row " + row );
                        popup.show(e.getComponent(), e.getX(), e.getY());
					}
					if( e.getClickCount() == 1 )
					{
						int row = TableOfCommObject.rowAtPoint( e.getPoint() );
						ObjTableModel otm = ( ObjTableModel ) TableOfCommObject.getModel();
						showProperties( ( String ) otm.getValueAt( row, 0 ) );
					}
				}
			} );
		    TableOfCommObject.registerKeyboardAction( (ActionListener)this, "deleteObject", KeyStroke.getKeyStroke( java.awt.event.KeyEvent.VK_DELETE, 0 ), JComponent.WHEN_FOCUSED );
		    TableOfCommObject.registerKeyboardAction( (ActionListener)this, "allProperties", KeyStroke.getKeyStroke( java.awt.event.KeyEvent.VK_ENTER, 0 ), JComponent.WHEN_FOCUSED );

    	    i = 0;
		    model = new PropertyTableModel( ListOfProperties );
		    TableOfCommProperties = new JTable( model );
/*		
		    TableColumn tc = null;
		    for ( int j = 0; j < 2; i++ ) 
		    {
                tc = TableOfCommProperties.getColumnModel().getColumn(j);
                if (j == 1) {
                      tc.setMinWidth(200); //sport column is bigger
                } else {
                      tc.setMinWidth(100);
                  }
            }
*/      
		    TableOfCommProperties.setPreferredScrollableViewportSize( new Dimension( 360, Math.min( 20 * i, 600 ) ) );
		    TableOfCommProperties.setSelectionMode( ListSelectionModel.SINGLE_SELECTION );
		    TableOfCommProperties.addMouseListener( new MouseAdapter()
			{
			    public void mouseClicked( MouseEvent e )
			    {
				    if( e.isMetaDown() )
				    {
					    int row = TableOfCommProperties.rowAtPoint( e.getPoint() );
					    //System.out.println( "right click at row " + row );
					    popup.show(e.getComponent(), e.getX(), e.getY());
				    }
			    }
			} );
	    	TableOfCommProperties.registerKeyboardAction( (ActionListener)this, "editProperty", KeyStroke.getKeyStroke( java.awt.event.KeyEvent.VK_ENTER, 0 ), JComponent.WHEN_FOCUSED );
    		//TableOfCommProperties.registerKeyboardAction( (ActionListener)this, "deleteProperty", KeyStroke.getKeyStroke( java.awt.event.KeyEvent.VK_DELETE, 0 ), JComponent.WHEN_FOCUSED );

	        valueField = new JTextField();
            valueField.setHorizontalAlignment(JTextField.LEFT);
            CommPropertyValueEditor = new DefaultCellEditor(valueField);
            TableOfCommProperties.setDefaultEditor(String.class, CommPropertyValueEditor);
            CommPropertyValueEditor.setClickCountToStart( 2 );
            CommPropertyValueEditor.addCellEditorListener( new CellEditorListener()
            {
                public void editingStopped(ChangeEvent e) 
                {
			        int selectedObj = TableOfCommObject.getSelectedRow();
			        int selectedProperty = TableOfCommProperties.getSelectedRow();
						
				    try
				    {
					    editProperty( selectedObj, selectedProperty, "CELL" );
				    }
				    catch( Exception ie )
				    {
					    JOptionPane.showMessageDialog( frame, "Unable to modify the data. " + ie.getMessage(), "Properties", JOptionPane.ERROR_MESSAGE );
				    }
                }
            
                public void editingCanceled(ChangeEvent e)
                {
                }
            } );
        
		    // System.out.println( " creating scroller..." );
		    // create the scroll pane and add the table to it
		    JScrollPane ObjectsScrollPane = new JScrollPane( TableOfCommObject );
		    JScrollPane PropertiesScrollPane = new JScrollPane( TableOfCommProperties );
		    
		    // add the scroll pane to the panel
		    panel = new JPanel();
		    panel.setLayout( new BoxLayout ( panel, BoxLayout.X_AXIS ) );
		    panel.add( ObjectsScrollPane, BorderLayout.WEST );
		    panel.add( PropertiesScrollPane, BorderLayout.CENTER );	        
		}

	    return panel;
    }

	private boolean closeUserInfoFile() throws IOException
	{
		if( changesMade )
		{
			// ask for save
			int result = JOptionPane.showConfirmDialog( frame,
														"Do you want to save the changes?",
														"Tango Data Manager",
														JOptionPane.YES_NO_CANCEL_OPTION );
			if( result == JOptionPane.YES_OPTION )
			{
				// save and quit
				updated = true;
				return true;
			}
			else if( result == JOptionPane.NO_OPTION )
			{
				// just quit w/o saving
				closed = true;
				return true;
			}
			else
			{
				// abort, cancel
				return false;
			}
		}
		
		// no change
		return true;
	}

	private void showProperties( String ObjName )
	{
		// table of properties
		// System.out.println( " creating table..." );
		
	    Enumeration properties = dm.getObjectKeys( tabTitle, ObjName );
	    int NumberOfProperties = 0;
	    while ( properties.hasMoreElements() )
	    {
	        properties.nextElement();
	        NumberOfProperties++;
	    }
	    
	    ListOfProperties = new Object[ NumberOfProperties ][ 2 ];
	    
		// System.out.println( "  filling table with the data..." );
		
		int i = 0;
        properties = dm.getObjectKeys( tabTitle, ObjName );
		while ( properties.hasMoreElements() )
		{
		    String property = properties.nextElement().toString();

	        ListOfProperties[ i ][ 0 ] = property;
	        ListOfProperties[ i ][ 1 ] = dm.getValue( tabTitle, ObjName, property );
		    i++;
		}

		// System.out.println( " DONE" );

	    if ( tabTitle.equals( "user" ) )
	    {
		    PropertyTableModel ptm = (PropertyTableModel)TableOfUserProperties.getModel();
		    ptm.recreate( ListOfProperties );
		    TableOfUserProperties.setPreferredScrollableViewportSize( new Dimension( 360, Math.min( 20 * i, 600 ) ) );
		}
		else if ( tabTitle.equals( "community" ) )
		{
		    PropertyTableModel ptm = (PropertyTableModel)TableOfCommProperties.getModel();
		    ptm.recreate( ListOfProperties );
		    TableOfCommProperties.setPreferredScrollableViewportSize( new Dimension( 360, Math.min( 20 * i, 600 ) ) );
		}
		
		tabbedPane.repaint();
	}

	private void addNewObject( String ObjName ) throws Exception
	{
		ObjTableModel otm = null;
		PropertyTableModel ptm = null;
		if ( tabTitle.equals( "user" ) )
		{
			otm = (ObjTableModel)TableOfUserObject.getModel();
			ptm = (PropertyTableModel)TableOfUserProperties.getModel();
		}
		else if ( tabTitle.equals( "community" ) )
		{
		    otm = (ObjTableModel)TableOfCommObject.getModel();
		    ptm = (PropertyTableModel)TableOfCommProperties.getModel();
		}
		
		boolean added = false;
		if ( otm.searchCell( ObjName ) == -1 )
		{
		    // if there's no object with the given name
		    dm.addObject( tabTitle, ObjName );
		    otm.addRow( ObjName );
		    added = true;
		}
		else
		{
		    // if the same name's object exists
			int i = JOptionPane.showConfirmDialog( frame, "The Name already exists. Do you want to overwrite?", "Create New Object", JOptionPane.YES_NO_OPTION );
			if ( i == JOptionPane.YES_OPTION )
			{
			    dm.addObject( tabTitle, ObjName );
				otm.removeRow( otm.searchCell( ObjName ) );
				otm.addRow( ObjName );
			    added = true;
			}
		}
		
		if ( added && tabTitle.equals( "user" ) )
		{
		    // get the mendatory properties for user object
		    DataEditor de = new DataEditor( frame, "Property Requirement", tabTitle, ObjName, '*' );
		    if ( de.getResult() )
		    {
		        dm.setValue( tabTitle, ObjName, "password", PasswdManager.getEncPasswd( de.getPassword() ) );
		        dm.setValue( tabTitle, ObjName, "realname", de.getRealName() );
		        dm.setValue( tabTitle, ObjName, "e-mail", de.getEmail() );
		        dm.setValue( tabTitle, ObjName, "community", de.getCommunity() );
		        dm.setValue( tabTitle, ObjName, "role", de.getRole() );
		    }
		}
		changesMade = true;
		frame.repaint();
	}
	
	private void deleteObject( int n ) throws Exception
	{
	    ObjTableModel otm = null;
	    PropertyTableModel ptm = null; 
	    
	    if ( tabTitle.equals( "user" ) )
	    {
		    otm = (ObjTableModel)TableOfUserObject.getModel();
		    ptm = (PropertyTableModel) TableOfUserProperties.getModel();
		}
		else if ( tabTitle.equals( "community" ) )
		{
		    otm = (ObjTableModel)TableOfCommObject.getModel();
		    ptm = (PropertyTableModel) TableOfCommProperties.getModel();
		}
		String ObjName = (String)otm.getValueAt( n, 0 );
		dm.removeObject( tabTitle, ObjName );
		otm.removeRow( n );
		changesMade = true;
		
		// clear the property table 
		ListOfProperties = new Object[ 0 ][ 0 ];
		ptm.recreate( ListOfProperties );
		frame.repaint();
	}

	private void showAllObjects()
	{
	    Enumeration objects = dm.getObjects( tabTitle );
	    int NumberOfObjects = 0;
	    while ( objects.hasMoreElements() )
	    {
	        objects.nextElement();
	        NumberOfObjects++;
	    }
	    
	    ListOfObjects = new Object[ NumberOfObjects ][ 1 ];
	    
		// System.out.println( "  filling table with the data..." );
		
		int i = 0;
		String ObjName = null;
        objects = dm.getObjects( tabTitle );
		while ( objects.hasMoreElements() )
		{
		    ObjName = objects.nextElement().toString();
	        ListOfObjects[ i ][ 0 ] = ObjName;
		    i++;
		}

		// System.out.println( " DONE" );

		if ( tabTitle.equals( "user" ) )
		{
    		ObjTableModel otm = (ObjTableModel)TableOfUserObject.getModel();
	    	otm.recreate( ListOfObjects );
		    TableOfUserObject.setPreferredScrollableViewportSize( new Dimension( 100, Math.min( 20 * i, 600 ) ) );
		
		    // clear the property table 
		    ListOfProperties = new Object[ 0 ][ 0 ];
		    PropertyTableModel ptm = (PropertyTableModel) TableOfUserProperties.getModel();
		    ptm.recreate( ListOfProperties );
		    TableOfUserProperties.setPreferredScrollableViewportSize( new Dimension( 360, Math.min( 20 * i, 600 ) ) );
		}
		else if ( tabTitle.equals( "community" ) )
		{
		    ObjTableModel otm = (ObjTableModel)TableOfCommObject.getModel();
		    otm.recreate( ListOfObjects );
		    TableOfCommObject.setPreferredScrollableViewportSize( new Dimension( 100, Math.min( 20 * i, 600 ) ) );
		
		    // clear the property table 
		    ListOfProperties = new Object[ 0 ][ 0 ];
		    PropertyTableModel ptm = (PropertyTableModel) TableOfCommProperties.getModel();
		    ptm.recreate( ListOfProperties );
		    TableOfCommProperties.setPreferredScrollableViewportSize( new Dimension( 360, Math.min( 20 * i, 600 ) ) );
		}
		
		tabbedPane.repaint();
	}

	private void deleteProperty( int n, int m ) throws Exception
	{
		ObjTableModel otm = null;
		PropertyTableModel ptm = null;
		
		if ( tabTitle.equals( "user" ) )
		{
		    otm = (ObjTableModel)TableOfUserObject.getModel();
		    ptm = (PropertyTableModel)TableOfUserProperties.getModel();
		}
		else if ( tabTitle.equals( "community" ) )
		{
		    otm = (ObjTableModel)TableOfCommObject.getModel();
		    ptm = (PropertyTableModel)TableOfCommProperties.getModel();
		}
		
	    String ObjName = (String)otm.getValueAt( n, 0 );
	    String propertyname = (String)ptm.getValueAt( m, 0 );
	    dm.removeKey( tabTitle, ObjName, propertyname );
	    ptm.removeRow( m );
	    changesMade = true;
	    frame.repaint();
	}

	private void editProperty( int n, int m, String type ) throws Exception
	{	    
		ObjTableModel otm = null;
		PropertyTableModel ptm = null;
		
		if ( tabTitle.equals( "user" ) )
		{
    		otm = (ObjTableModel)TableOfUserObject.getModel();
	    	ptm = (PropertyTableModel)TableOfUserProperties.getModel();
		}
		else if ( tabTitle.equals( "community" ) )
		{
		    otm = (ObjTableModel)TableOfCommObject.getModel();
		    ptm = (PropertyTableModel)TableOfCommProperties.getModel();
		}
		
	    String ObjName = (String)otm.getValueAt( n, 0 );
	    String propertyname = (String)ptm.getValueAt( m, 0 );
	    String propertyvalue = (String)ptm.getValueAt( m, 1 );
	    
	    if (type.equals( "CELL" ))
	    {
	        String cellvalue = null;
	        if ( tabTitle.equals( "user" ) )
	        {
	            cellvalue = (String)UserPropertyValueEditor.getCellEditorValue();
	        }
	        else if ( tabTitle.equals( "community" ) )
	        {
	            cellvalue = (String)CommPropertyValueEditor.getCellEditorValue();
	        }
	        dm.setValue( tabTitle, ObjName, propertyname, cellvalue );
	        ptm.setValueAt( cellvalue, m, 1 );
	        changesMade = true;
	        frame.repaint();
        }
        else if (type.equals( "GUI" ))
        {
	        DataEditor ue = new DataEditor( frame, "Edit Property", ObjName, propertyname, propertyvalue );
	        if (ue.getResult())
	        {
	            dm.setValue( tabTitle, ObjName, propertyname, ue.getPropertyValue() );
    	        ptm.setValueAt( ue.getPropertyValue(), m, 1 );
	            changesMade = true;
	            frame.repaint();
	        }  
	    }
	}

    private void addNewProperty( int n ) throws Exception
    {
		ObjTableModel otm = null;
		PropertyTableModel ptm = null;
        
        if ( tabTitle.equals( "user" ) )
        {
		    otm = (ObjTableModel)TableOfUserObject.getModel();
		    ptm = (PropertyTableModel)TableOfUserProperties.getModel();
        }
        else if ( tabTitle.equals( "community" ) )
        {
		    otm = (ObjTableModel)TableOfCommObject.getModel();
		    ptm = (PropertyTableModel)TableOfCommProperties.getModel();
        }
        
		String ObjName = (String)otm.getValueAt( n, 0 );
        
		DataEditor ue = new DataEditor( frame, "Add New Property", ObjName );
		if( ue.getResult() )
		{
			// modify data
			dm.setValue( tabTitle, ObjName, ue.getPropertyName(), ue.getPropertyValue() );
			int i = ptm.searchCell( ue.getPropertyName() );
            if ( i == -1 )
            {
               Object[] newData = { ue.getPropertyName(), ue.getPropertyValue() };
               ptm.addRow( newData );
            }
            else
            {
			   ptm.setValueAt( ue.getPropertyValue(), i, 1 );
			}
		    changesMade = true;
		    frame.repaint();
		}
    }

    private void addPassword( int n ) throws Exception
    {
		ObjTableModel otm = null;
		PropertyTableModel ptm = null;
		if ( tabTitle.equals( "user" ) )
		{
		    otm = (ObjTableModel)TableOfUserObject.getModel();
		    ptm = (PropertyTableModel)TableOfUserProperties.getModel();
		}
		else if (tabTitle.equals( "community" ) )
		{
    		otm = (ObjTableModel)TableOfCommObject.getModel();
	    	ptm = (PropertyTableModel)TableOfCommProperties.getModel();
		}
		
		String ObjName = (String)otm.getValueAt( n, 0 );
		DataEditor ue = new DataEditor( frame, "Add New Password", ObjName, '*' );
		if( ue.getResult() )
		{
			// modify data
			int i = ptm.searchCell( "password" );
			String encPasswd = PasswdManager.getEncPasswd( ue.getPassword() );
			dm.setValue( tabTitle, ObjName, "password", encPasswd );
            if ( i == -1 )
            {
               Object[] newData = { "password", encPasswd };
               ptm.addRow( newData );
            }
            else
            {
			   ptm.setValueAt( encPasswd, i, 1 );
			}
		    changesMade = true;
		    frame.repaint();
		}
    }
}
