//=============================================================================
//
// class ObjTableModel
//
// Author:			Grzegorz Lewandowski
// Created:			12/02/98
// Original class:  PasswordTableModel
//
// Modified:		07/20/99
// By:				Hyun Kim
// Changes:			name changed from PasswordTableModel to ObjTableModel
// Description:     display the list of Tango users and communities
//
//=============================================================================

package webwisdom.tango.config;

import com.sun.java.swing.*;
import com.sun.java.swing.table.*;

import java.awt.*;
import java.awt.event.*;

/**Table model for displaying objects.
 */
public class ObjTableModel extends AbstractTableModel
{
	Object[][] TableOfObjects;
	Object[] columnNames = { "Name" };

	public ObjTableModel( Object[][] d )
	{
		TableOfObjects = d;
			//{{INIT_CONTROLS
		//}}
    }

	public void recreate( Object[][] d )
	{
		TableOfObjects = d;
        fireTableStructureChanged();		
		fireTableDataChanged();
	}

	public int getRowCount()
	{
		return TableOfObjects.length;
	}

	public int getColumnCount()
	{
		return columnNames.length;
	}
	
	public String getColumnName( int col )
	{
		return (String)columnNames[ col ];
	}
	
	public Object getValueAt( int row, int col )
	{
		return TableOfObjects[ row ][ col ];
	}
	
	public String getColumnNames( int col )
	{
	    return (String) columnNames[ col ];
	}
	
	public int searchCell( Object value )
	{	    
	    for (int i = 0; i < TableOfObjects.length; i++)
	    {
	        if (TableOfObjects[ i ][ 0 ].equals( value ))
	        {
	            return i;
	        }
	    }
	    return -1;
	}

	public Class getColumnClass( int c )
	{
		return getValueAt( 0, c ).getClass();
	}

	public void setValueAt( Object value, int row )
	{
		TableOfObjects[ row ][ 0 ] = value;
		fireTableCellUpdated( row, 0 );
	}

	public void addRow( Object rowData )
	{
		Object[][] newData = new Object[ TableOfObjects.length + 1 ][ 1 ];
		for( int i = 0; i < TableOfObjects.length; i++ )
		{
			newData[ i ] = TableOfObjects[ i ];
		}
		newData[ TableOfObjects.length ][ 0 ] = rowData;
		TableOfObjects = newData;
		fireTableRowsInserted( TableOfObjects.length, TableOfObjects.length );
	}

	public void removeRow( int rowIndex )
	{
		if( rowIndex >= TableOfObjects.length )
		{
			return;
		}
		Object[][] newData = new Object[ TableOfObjects.length - 1 ][ 1 ];
		int i = 0;
		int j = 0;
		while( i < TableOfObjects.length )
		{
			if( i == rowIndex )
			{
				i++;
				continue;
			}
			newData[ j++ ] = TableOfObjects[ i++ ];
		}
		TableOfObjects = newData;
		fireTableRowsDeleted( rowIndex, rowIndex );
	}
	//{{DECLARE_CONTROLS
	//}}
}
