//=============================================================================
//
// class PropertyTableModel
//
// Author:			Grzegorz Lewandowski
// Created:			12/02/98
// Original class:  PasswordTableModel
//
// Modified:		07/20/99
// By:				Hyun Kim
// Changes:			name changed from PasswordTableModel to PropertyTableModel
// Description:     display properties of Tango users and communities
//
//=============================================================================

package webwisdom.tango.config;

import com.sun.java.swing.*;
import com.sun.java.swing.table.*;

import java.awt.*;
import java.awt.event.*;

/**Table model for displaying properties.
 */
public class PropertyTableModel extends AbstractTableModel
{
	Object[][] TableOfProperties;
	Object[] columnNames = { "Property", "Value" };

	public PropertyTableModel( Object[][] d )
	{
		TableOfProperties = d;
			//{{INIT_CONTROLS
		//}}
    }

	public void recreate( Object[][] d )
	{
		TableOfProperties = d;
        fireTableStructureChanged();		
		fireTableDataChanged();
	}

	public int getRowCount()
	{
		return TableOfProperties.length;
	}

	public int getColumnCount()
	{
		return columnNames.length;
	}

	public String getColumnName( int col )
	{
		return (String)columnNames[ col ];
	}
	
	public Object getValueAt( int row, int col )
	{
		return TableOfProperties[ row ][ col ];
	}

	public Class getColumnClass( int c )
	{
		return getValueAt( 0, c ).getClass();
	}

	public void setValueAt( Object value, int row, int col )
	{
		TableOfProperties[ row ][ col ] = value;
		fireTableCellUpdated( row, col );
	}
    
    public boolean isCellEditable(int row, int col) {
        //Note that the data/cell address is constant,
        //no matter where the cell appears onscreen.
        if (col < 1) 
        { 
            return false;
        } 
        else 
        {
            return true;
        }
    }
    
	public int searchCell( Object value )
	{	    
	    for (int i = 0; i < TableOfProperties.length; i++)
	    {
	        if (TableOfProperties[ i ][ 0 ].equals( value ))
	        {
	            return i;
	        }
	    }
	    return -1;
	}

	public void addRow( Object[] rowData )
	{
		Object[][] newData = new Object[ TableOfProperties.length + 1 ][ 2 ];
		for( int i = 0; i < TableOfProperties.length; i++ )
		{
			newData[ i ] = TableOfProperties[ i ];
		}
		newData[ TableOfProperties.length ] = rowData;
		TableOfProperties = newData;
		fireTableRowsInserted( TableOfProperties.length, TableOfProperties.length );
	}

	public void removeRow( int rowIndex )
	{
		if( rowIndex >= TableOfProperties.length )
		{
			return;
		}
		Object[][] newData = new Object[ TableOfProperties.length - 1 ][ 2 ];
		int i = 0;
		int j = 0;
		while( i < TableOfProperties.length )
		{
			if( i == rowIndex )
			{
				i++;
				continue;
			}
			newData[ j++ ] = TableOfProperties[ i++ ];
		}
		TableOfProperties = newData;
		fireTableRowsDeleted( rowIndex, rowIndex );
	}
	//{{DECLARE_CONTROLS
	//}}
}
