//=============================================================================
//
// class ProtocolBox
//
// Author:			Grzegorz Lewandowski
// Created:			12/02/98
// Original class:  UserEditor
//
// Modified:		07/20/99
// By:				Hyun Kim
// Changes:			name changed from UserEditor to ProtocolBox
// Description:     create GUI for protocol setup
//
//=============================================================================

package webwisdom.tango.config;

import com.sun.java.swing.*;

import java.awt.*;
import java.awt.event.*;

/**This class allows creating a GUI for opening the protocol.
 */
public class ProtocolBox implements ActionListener
{
	private JDialog frame;

	private JLabel idLabel, pwdLabel, urlLabel;
	private JTextField urlField;
	private JPasswordField idField, pwdField;

	private String loginID, password, myURL;
	private boolean result, isURL = false;

	/**Constructs ProtocolBox object for connecting to database.
	 * @param title title for ProtocolBox window
	 */
	public ProtocolBox( Frame owner, String title )
	{
		createGUI( owner, title );
			//{{INIT_CONTROLS
		//}}
    }

	/**Constructs ProtocolBox object for http connection.
	 * @param title title for ProtocolBox window
	 * @param type type for http connection
	 */
	public ProtocolBox( Frame owner, String title, String type )
	{
	    isURL = true;
		createGUI( owner, title, type );
	}
	
	//
	// ActionListener
	//
	public void actionPerformed( ActionEvent e )
	{
		String command = e.getActionCommand();
		if( command.compareTo( "OK" ) == 0 )
		{
			result = true;
			if ( isURL )
			{
		        myURL = urlField.getText();
		    }
		    else
		    {
			    loginID = idField.getText();
		        password = pwdField.getText();
		        myURL = urlField.getText();
		    }
			frame.setVisible( false );
		}
		else if( command.compareTo( "Cancel" ) == 0 )
		{
			result = false;
			frame.setVisible( false );
		}
	}

	public boolean getResult()
	{
		return result;
	}

	public String getID()
	{
		return loginID;
	}

	public String getPassword()
	{
		return password;
	}
	
	public String getURL()
	{
	    return myURL;
	}

	//
	// private methods
	//
    
	void createGUI( Frame owner, String title )
	{
	    // create GUI for connecting to database
	    
		frame = new JDialog( owner, title, true );

		idLabel = new JLabel( "Login ID:" );
		pwdLabel = new JLabel( "Password:" );
		urlLabel = new JLabel("URL:");

		idField = new JPasswordField( loginID, 20 );
		idField.setEchoChar('*');
		idField.setText( loginID );
		idField.requestFocus();
		pwdField = new JPasswordField( password, 20 );
		pwdField.setEchoChar('*');
		pwdField.setText( password );
		urlField = new JTextField( myURL, 25 );
		//urlField.setText( "jdbc:oracle:thin:@carver.npac.syr.edu:1521:europe" );
		//urlField.setText( "jdbc:odbc:syntaxTest" );
		idField.setText( "PIM" );
		pwdField.setText( "PIM" );
		urlField.setText( "jdbc:oracle:thin:@leccorder.npac.syr.edu:1521:ORCL" );
		
		// layout the labels in a panel
		JPanel labelPane = new JPanel();
		labelPane.setLayout( new GridLayout( 0, 1 ) );
		labelPane.add( idLabel );
		labelPane.add( pwdLabel );
		labelPane.add( urlLabel );

		// layout the text fields in a panel
		JPanel fieldPane = new JPanel();
		fieldPane.setLayout( new GridLayout( 0, 1 ) );
		fieldPane.add( idField );
		fieldPane.add( pwdField );
		fieldPane.add( urlField );

		// layout the buttons in a panel
		JPanel buttonPane = new JPanel();
		buttonPane.setBorder( BorderFactory.createEmptyBorder( 5, 10, 5, 10 ) );
		buttonPane.setLayout( new GridLayout( 1, 0, 10, 5 ) );
		JLabel l = new JLabel( "" );
		buttonPane.add( l );
		JButton button = new JButton( "OK" );
		button.addActionListener( this );
		button.setVerticalTextPosition( AbstractButton.CENTER );
        button.setHorizontalTextPosition( AbstractButton.CENTER );
		buttonPane.add( button );
		button = new JButton( "Cancel" );
		button.addActionListener( this );
		button.setVerticalTextPosition( AbstractButton.CENTER );
        button.setHorizontalTextPosition( AbstractButton.CENTER );
		buttonPane.add( button );
		l = new JLabel( "" );
		buttonPane.add( l );

		// put the panels in another panel, labels on left,
		// text fields on right
		JPanel contentPane = new JPanel();
		contentPane.setBorder( BorderFactory.createEmptyBorder( 20, 20, 5, 20 ) );
		contentPane.setLayout( new BorderLayout() );
		contentPane.add( labelPane, BorderLayout.CENTER );
		contentPane.add( fieldPane, BorderLayout.EAST );
		contentPane.add( buttonPane, BorderLayout.SOUTH );

		frame.setContentPane( contentPane );
		frame.pack();
		frame.setVisible( true );
	}
	
	void createGUI( Frame owner, String title, String type )
	{
	    // create GUI for connecting to the web server(http connection)
	    
		frame = new JDialog( owner, title, true );

		urlLabel = new JLabel( "URL:" );

		urlField = new JTextField( myURL, 25 );
		urlField.setText( "http://trurl.npac.syr.edu/hkim/userdata.dat" );
		urlField.requestFocus();
		
		// layout the labels in a panel
		JPanel labelPane = new JPanel();
		labelPane.setLayout( new GridLayout( 0, 1 ) );
		labelPane.add( urlLabel );

		// layout the text fields in a panel
		JPanel fieldPane = new JPanel();
		fieldPane.setLayout( new GridLayout( 0, 1 ) );
		fieldPane.add( urlField );

		// layout the buttons in a panel
		JPanel buttonPane = new JPanel();
		buttonPane.setBorder( BorderFactory.createEmptyBorder( 5, 10, 5, 10 ) );
		buttonPane.setLayout( new GridLayout( 1, 0, 10, 5 ) );
		JLabel l = new JLabel( "" );
		buttonPane.add( l );
		JButton button = new JButton( "OK" );
		button.addActionListener( this );
		button.setVerticalTextPosition( AbstractButton.CENTER );
        button.setHorizontalTextPosition( AbstractButton.CENTER );
		buttonPane.add( button );
		button = new JButton( "Cancel" );
		button.addActionListener( this );
		button.setVerticalTextPosition( AbstractButton.CENTER );
        button.setHorizontalTextPosition( AbstractButton.CENTER );
		buttonPane.add( button );
		l = new JLabel( "" );
		buttonPane.add( l );

		// put the panels in another panel, labels on left,
		// text fields on right
		JPanel contentPane = new JPanel();
		contentPane.setBorder( BorderFactory.createEmptyBorder( 20, 20, 5, 20 ) );
		contentPane.setLayout( new BorderLayout() );
		contentPane.add( labelPane, BorderLayout.CENTER );
		contentPane.add( fieldPane, BorderLayout.EAST );
		contentPane.add( buttonPane, BorderLayout.SOUTH );

		frame.setContentPane( contentPane );
		frame.pack();
		frame.setVisible( true );
	}	
	//{{DECLARE_CONTROLS
	//}}
}