/*
File: WatcherMessage.java
// $Id: WatcherMessage.java,v 1.1 1999/07/13 21:23:19 beca Exp $
Component: Plugin
Date		Author		Changes
Jun 30 99	Lukasz Beca	Created
*/


package webwisdom.tango.messages;

import webwisdom.tango.messages.Message;
import webwisdom.tango.consts.Const;
import webwisdom.tango.threads.Watcher;
import java.io.*;



/**
 * A class representing message that is used in communication with PeopleWatch 
 * 
 *@author Lukasz Beca
 *@see Interfaces in CIV Communication System documentation
 **/
public class WatcherMessage extends Message
{
    private static final String CL="WatcherMessage";

    private byte watcherType;
    private int  len;
    private byte data[];
  
    public WatcherMessage(byte typepar,byte datapar[])
    {
      type = Const.WATCHER;
      
      watcherType=typepar;
      len=datapar.length;
      data=new byte[len];
      
      for(int i=0;i<len;i++)
      {
	data[i]=datapar[i];
      }
    }// AppEventMessage ____________________________________________________


    /**Creates a new EVENT message from the stream
     *@param instream message data*/
    WatcherMessage(DataInputStream instream) throws IOException
    {
      type=instream.readInt();
      if(type==Const.WATCHER)
      {
	    len=instream.readInt() - 1;
	    watcherType = instream.readByte();
	    data=new byte[len];         
	    instream.readFully(data,0,len);	   
	    
      } else {
        
        System.err.println(CL+".WatcherMessage(): type="+type+
			   " not supported!: creating null message");
        type=Const.UNDEFINED;
        watcherType = Watcher.UNDEFINED;
        len=0;
        data=null;
      }
    }// AppEventMessage ____________________________________________________


    /**Writes EVENT message to the output stream
     *@param outstream output stream*/
    public void send(DataOutputStream outstream) throws IOException
    {
      //System.out.println(CL+"@"+hashCode()+".send()");
      outstream.writeInt(type);
      outstream.writeInt(len + 1);
      outstream.writeByte(watcherType);      
      outstream.write(data,0,len);
      outstream.flush();
      //System.out.println(CL+"@"+hashCode()+".send().");
    }// send _______________________________________________________________
  

    /**
     *@return returns length of table containing information*/
    public int getLen()
    {
      return len;
    }// getLen _____________________________________________________________   


    /**
     *@return returns reference to the table with information*/
    public byte[] getData()
    {
      return data;
    }// getDate ____________________________________________________________

    public byte getWatcherType() {
            
        return watcherType;
    }


    public String toString()
    {
      return CL+"@"+hashCode()+"["+Const.toString(type)+",byte["+len+"]]";
    }// toString ___________________________________________________________


}// AppEventMessage ////////////////////////////////////////////////////////


