// $Id: PasswdManager.java,v 1.14 1999/10/29 17:56:47 beca Exp $
/*
File: PasswdManager.java
Component: Server
Date		Author		Changes
Nov 30 98	Lukasz Beca	Created
*/

package webwisdom.tango.server;
 
import java.io.*;
import java.util.*;
import webwisdom.tango.test.Log;
import webwisdom.tango.config.*;

public class PasswdManager {  
  
  private dataModule access;
  
  private static PasswdManager instance = null;
           
  public static PasswdManager getInstance() {               
        
        return instance;            
  } 
  
  private PasswdManager() {
        
  }
  
  public static void init(String url, String user, String password) throws Exception {
  
    instance = new PasswdManager();
    instance.setup(url, user, password);      
  }
    
  private void setup(String url, String user, String password) throws Exception {
           
      access = new dataModule();  
      access.open(getProtocol(url), getLocation(url), user, password);
  }

  //close() Ends session with PasswdManager class without storing modifications in the file.
      
  public void close() {
   
      access.close();      	
  }  			
  
  /*getPassword(String name ) Retrieves password of the 'name' user. If no password is
assigned, string of length 0 is returned (not null). If the
user 'name' is not present in the file or entry
format is invalid, null is returned. */
  
  public String getPassword(String name) {
  
      String encPasswd;
            
      encPasswd = access.getValue("user", name, "password");
                  
      return encPasswd;
  
  }
  
  /*  public String getStatus(String name) 

Retrieves status of the 'name' user. If no status is
assigned, string of length 0 is returned (not null). If the
user 'name' is not present in the file or entry
format is invalid, null is returned.
  */
  
  public String getStatus(String name) {
  
      String status;
            
      status = access.getValue("user", name, "status");
                  
      return status;
     
  }
  
  // following methods behave in similar fashion as  getStatus(String name)
  
  public String getFullName(String name) {
  
      String fullName;
            
      fullName = access.getValue("user", name, "fullname");
                  
      return fullName;
     
  }
  
  public String getEMail(String name) {
  
      String eMail;
            
      eMail = access.getValue("user", name, "email");
                  
      return eMail;
     
  }
  
  /*
  public Hashtable getPasswordEntries()
  
  Returns Hashtable with all valid password entries in the 
file. Each entry is of the form: (name, encoded_password). 
Name is the key in the hash table. Method returns null if
the password file does not exist.
*/
  
  public Hashtable getPasswordEntries() {
  
    Hashtable result;
    
    result = getEntries("password");
      
    return result;

  }

// following methods behave in similar fashion as getPasswordEntries() 
 
  public Hashtable getStatusEntries() {
  
    Hashtable result;
    
    result = getEntries("status");
      
    return result;

  }
  
  public Hashtable getFullNameEntries() {
  
    Hashtable result;
    
    result = getEntries("fullname");
      
    return result;

  }


  public Hashtable getEMailEntries() {
  
    Hashtable result;
    
    result = getEntries("email");
      
    return result;

  }
  
  
  public void modifyPassword(String name, String value) {
        
    modifyEntry(name, "password", value);      
  }
  
  public void modifyStatus(String name, String value) {
    
    modifyEntry(name, "status", value);        
  }
  
  public void modifyFullName(String name, String value) {
    
    modifyEntry(name, "fullname", value);        
  }
  
  public void modifyEMail(String name, String value) {
        
    modifyEntry(name, "email", value);    
  }    
  
  /*  
  
  public boolean passwordOK(String name, String decPasswd) 
  
     Checks if the password is valid for 'name' user. Password
should be provided in decoded form e.g. "hammer" or "elephant",
not 
"-17.-75.-88.98.-80.101.-98.52.76.91.-111.14.120.69.-22.-68.15.104.59.125"
Returns 'true' if the password is valid, 'false' otherwise. 
    
 */   
  
  public boolean passwordOK(String name, String decPasswd) {
  
      String passwdFromFile;      
      boolean result = false;	    
		        
      passwdFromFile = getPassword(name);
                           
      if (decPasswd == null || passwdFromFile == null) {
      
	       return false;
	
	  } 
           
      if (decPasswd.length() == 0 && passwdFromFile.length() == 0) {
      
	      result = true;
	      
      } else {                   
      
          result = Password.passwordOK(decPasswd, passwdFromFile);        
      }   
      
      return result;          
  }
  
  /*
    
    public void addEntry(String name, String decPasswd, String status, String fullName, String eMail) throws IOException 
  
Updates data about users stored in the memory. Entry has the format:

name:encoded_password:status:full_name:e_mail

If the entry for 'name' user already exists in the file, the content 
of the entry is overwritten. 
If the entry for 'name' user does not exist, the new entry is added in the
file. 

Special case: it is possible to assign empty ("") password and
status to the user. 

*/
  
  
  public void addEntry(String name, String decPasswd, String status, String fullName, String eMail) {  
  
	   String encPasswd;
	   	
	   // check if all fields are valid
	         
     if (name == null || decPasswd == null || status == null || fullName == null || eMail == null) {
     
       return;
     
     }
         
	 	         
	 // get encrypted version of password
		 
     encPasswd = Password.getEncPasswd(decPasswd);          
      
     System.out.println("newLine =" +  name + ":" + encPasswd + ":" + status + ":" + fullName + ":" + eMail);
     		             
	 try {
	 
	    access.addObject("user", name);
	    access.setValue("user", name, "password", encPasswd);
	    access.setValue("user", name, "status", status);
	    access.setValue("user", name, "fullname", fullName);
	    access.setValue("user", name, "email", eMail);
	 } catch(Exception e) {
	    
	    e.printStackTrace(Log.err);  
	 }	 	   								  																																																 																																																																																					 							       																								                                                                         
  }
  
  
  
/*
public void removeEntry(String name) throws IOException
  
Removes user entry from the memory. 
*/  
  
   public void removeEntry(String name) {
  	   
     if (name == null) {
     
       return;
     
     }
     
     // entry is removed from the hash table
        
     try {
        access.removeObject("user", name);	 	         																																																																											 							       																								                                                                         
     } catch(Exception e) {
                
        e.printStackTrace(Log.err);  
     }
  }           
  
  public Hashtable getEntries(String field) {
  
    String line, user, data = null;
    Hashtable result = new Hashtable();
    Enumeration names = access.getObjects("user");
 
    while(names.hasMoreElements()) {
      
      // read line
      
      user = (String) names.nextElement();                       
		      			  		      			     			      
		data = (String) access.getValue("user", user, field);				    				    				    												    																    
																												    				    				    
		if (data == null) {
				    
			Log.out.println("PasswdManager: null data at field" + field, 2);
			data = new String("");
		}
				    			    
		// put data in hashtable
							    
		result.put(user, data);
		data = null;				      								  			         
	   //  System.out.println("PasswdManager: user added with parameters " + user + " " + password);            	    
    } 
        
    return result;
  }
  
  /*  
      
   public void update() throws IOException  

   Writes the data stored in the hashtable to the file 
      
   */  
  
  
  public void update() throws IOException {       
		
	access.update();
  }
  
  
    /*  
      
   public void update() throws IOException  

   Writes the data stored in the hashtable to the 'filename' file
      
   */  
  
  
  public void update(String filename) throws IOException {       
	
	access.update(filename);
  }

  private void modifyEntry(String name, String field, String newValue) {
       
    String value, newLine, decryptedPassword;
    int i;
    
    if (field.equals("password")) {
        
        decryptedPassword = newValue;
        newValue = Password.getEncPasswd(decryptedPassword);        
        
    } 
    
    try {
    
        access.setValue("user", name, field, newValue);
        
    } catch(Exception e) {
       
       e.printStackTrace(Log.err);       
           
    }
    
  }   
  
  
  public static String getProtocol(String url) {
    
    String result;
    int slashPosition;
    
    slashPosition = url.indexOf("://");
    
    if (slashPosition > 0) {
        
        result = url.substring(0, slashPosition);
        
        if (result.equals("http")) result = "web";
        
    } else {
        
        result = "file";   
    }
    
    return result; 
  }
  
  public static String getLocation(String url) {
    
    String result;
    int slashPosition;
    
    slashPosition = url.indexOf("://");
    
    if (slashPosition > 0 && !getProtocol(url).equals("web")) {
        
        result = url.substring(slashPosition + 3);
    } else {
        
        result = url;   
    }
    
    return result;     
  }
}
