/***************************************************************************
  $Id: TangoLicense.java,v 1.1 1999/07/30 20:24:14 beca Exp $
  Author: LukaszBeca, 98/04/16 
  Component: Server
***************************************************************************/


package webwisdom.tango.server;


import java.io.*;
import java.net.*;
import java.util.*;

 
/**Tango server.*/
public final class TangoLicense
{
    private static final String CL="TangoLicense";

    public static void main(String args[]) {
        
        Parameters p = new Parameters(args);
     
        
        if (p.error()) {
            
           // p.errorMessage();
                        
        } else {
            
            switch(p.getType()) {
                
                case License.UNLIMITED:
                    generateUnlimitedLicense();
                    break;
                
                case License.COMMERCIAL:
                    if (p.noIP()) {
                        
                        generateSimpleNoIPLicense(p.getUsers());    
                        
                    } else {
                        
                        generateSimpleLicense(p.getUsers(), p.getIP());                        
                    }
                    break;
                
                case License.GENERAL:
                    if (p.noIP()) {
                        
                        generateDateNoIPLicense(p.getUsers(), p.getStartMonth(), p.getStartYear(), p.getInterval());    
                        
                    } else {
                        
                        generateDateLicense(p.getUsers(), p.getStartMonth(), p.getStartYear(), p.getInterval(), p.getIP());                        
                    }
                    break;
                    
                case Parameters.HELP:
                    break;
                                    
                default:
                    Parameters.errorMessage();
                
            }                                    
        }                               

    }// main _______________________________________________________________
    
    
    public static void generateUnlimitedLicense() {
        
        LicenseGenerator lg = new LicenseGenerator();
        System.out.println(lg.getUnlimitedLicense());
        
    }
        
    public static void generateSimpleLicense(int users, String ip) {
                        
        LicenseGenerator lg = new LicenseGenerator();
        
        if (users > 0 && users < 101 &&
            !ip.equals("0.0.0.0")) {
        
            System.out.println(lg.getCommercialLicense(users, ip));
            
        } else {
            
            Parameters.errorMessage();   
        }
        
    }
    
    public static void generateSimpleNoIPLicense(int users) {

       LicenseGenerator lg = new LicenseGenerator(); 
       if (users > 0 && users < 101) {
       
            System.out.println(lg.getNoIPCommercialLicense(users)); 
            
       } else {
            
            Parameters.errorMessage();   
       }        
    }
    
    public static void generateDateLicense(int allowedUsers, int startMonth, int year, int number, String ip) {
        
       LicenseGenerator lg = new LicenseGenerator(); 
       
       if (allowedUsers > 0 && allowedUsers < 101 &&
           startMonth >= 0 && startMonth < 12 &&
           year > 0 &&
           number > 0 && number < 14 &&
           !ip.equals("0.0.0.0")) {
       
            System.out.println(lg.getGeneralLicense(allowedUsers, startMonth, year, number, ip));
            
       } else {
        
          Parameters.errorMessage();
       }
        
    }
    
    public static void generateDateNoIPLicense(int allowedUsers, int startMonth, int year, int number) {
        
        LicenseGenerator lg = new LicenseGenerator();
        
         if (allowedUsers > 0 && allowedUsers < 101 &&
           startMonth >= 0 && startMonth < 12 &&
           year > 0 &&
           number > 0 && number < 14) {
        
            System.out.println(lg.getNoIPGeneralLicense(allowedUsers, startMonth, year, number));
            
         } else {
        
            Parameters.errorMessage();
          
        }        
    }
        
}// TangoServer ////////////////////////////////////////////////////////////

class Parameters {
    
    public static final int HELP = 5;
    private boolean noip;
    private int type;
    private int users;
    private int startMonth;
    private int startYear;
    private int interval;
    private String ip;
    private boolean error;       
    
    public Parameters(String args[]) {
        
        GregorianCalendar now = new GregorianCalendar();        
        noip = false;
        error = false;
        type = License.INVALID;
        users = 0;
        startMonth = now.get(GregorianCalendar.MONTH);;
        startYear = now.get(GregorianCalendar.YEAR);
        ip = "0.0.0.0";
        interval = 0;        
        
        interpret(args);
        
    }
    
    public boolean noIP(){
        
        return noip;
    }
    
    public String getIP() {
        
        return ip;
        
    }    
    
    public int getType() {
        
        return type;
        
    }    
    
    public int getUsers() {
        
        return users;
    }
    
    public int getStartMonth() {
        
        return startMonth;
    }
    
    public int getStartYear() {
        
        return startYear;
    }
    
    public int getInterval() {
        
        return interval;
    }
    
    public boolean error() {
        
        return error;   
    }
    
    public static void errorMessage() {
        
       //print info about error and show valid parameters   
       System.out.println("Invalid parameters");
       helpMessage();
        
    }
    
    public static void helpMessage() {
       
       System.out.println("Usage:"); 
        
       System.out.println("java webwisdom.tango.server.TangoLicense options"); 
       
       System.out.println("Options:");
       System.out.println("-u : unlimited license");
       
       System.out.println("-s -un users -ip IPaddress : license without expiration date");
       
       System.out.println("-s -noip -un users : license witout expiration date and no ip");
       
       System.out.println("-d -un users -sy startyear -sm startmonth -m months -ip IPaddress : license with expiration date");
       
       System.out.println("-d -noip -un users -sy startyear -sm startmonth -m months : license with expiration date and no ip");
       
       System.out.println("-h : help");
        
    }
    
    private void interpret(String args[]) {
     
     int argNumber = args.length;   
     int i;
        
     
     for (i = 0; i < argNumber; i ++) {   
        
        if (args[i].equals("-u")) {
                
                type = License.UNLIMITED;
            
            } else if (args[i].equals("-s")) {
                
                type = License.COMMERCIAL;
                
            } else if (args[i].equals("-d")) {
                            
                type = License.GENERAL;                                   
                
            } else if (args[i].equals("-noip")) {
                
                noip = true;               
            
            } else if (args[i].equals("-un")) {
                
                if (i < argNumber) {
                
                    users = Integer.parseInt(args[i + 1]);   
                    i ++;
                } else {
                    
                    errorMessage();   
                    
                }
            
            } else if (args[i].equals("-ip")) {
                
                if (i < argNumber) {
                    ip = args[i + 1];
                    i ++;
                } else {
                    
                    errorMessage();   
                    
                }
            } else if (args[i].equals("-sm")) {
                
                if (i < argNumber) {
                    startMonth = Integer.parseInt(args[i + 1]) - 1;
                    i ++;
                } else {
                    
                    errorMessage();   
                    
                }
            } else if (args[i].equals("-sy")) {
                
                if (i < argNumber) {
                    startYear = Integer.parseInt(args[i + 1]);
                    i ++;
                } else {
                    
                    errorMessage();   
                    
                }
            } else if (args[i].equals("-m")) {
                
                if (i < argNumber) {                
                    interval = Integer.parseInt(args[i + 1]);
                    i ++;
                } else {
                    
                    errorMessage();   
                    
                }
            } else if (args[i].equals("-h")) {
                
                type = HELP;
                helpMessage();   
                
            } else {
                
                errorMessage();  
                error = true;
                
            }
        }    
    }    
}
