// $Id: CentralServer.java,v 1.12 1999/10/28 17:30:33 remek Exp $
/*
File: CentralServer.java
Component: Plugin
Date		Author		Changes
Mar 07 97	Lukasz Beca	Created
*/

package webwisdom.tango.structures;

import webwisdom.tango.structures.ControlApplication;
import webwisdom.tango.structures.LocalApplications;
import webwisdom.tango.threads.DemonThreadIn;
import webwisdom.tango.threads.ThreadOut;
import webwisdom.tango.messages.*;
import java.net.Socket;
import java.io.*;
import java.util.*;
import webwisdom.tango.consts.Const;
import webwisdom.tango.test.Log;
import netscape.security.PrivilegeManager;


public class CentralServer
{
    private static final String CL="CentralServer";


    private ControlApplication ca;
    private LocalApplications apps;
    private DemonThreadIn in;
    private ThreadOut out;
    private String serverhost;
    private int serverport;


    public CentralServer()
    {
      ca = null;
      apps = null;
      in = null;
      out = null;
      serverhost = null;
      serverport = 0;
    }


    //interface for LocalBase
    //public void initAssociations(ControlApplication capar,LocalApplications lapar);
 

    public void initAssociations(ControlApplication capar,
				 LocalApplications lapar)
    {
      ca = capar;
      apps = lapar;
//      initSocket();
    }
   

    //interface for ControlApplication
    //public void receiveFromCA(Message m);
    //public void logout();
    //public String getServerHost();


    public void sendCAToCS(Message m)
    {
      out.putMessage(m);
    }
  

    public String getServerHost()
    {
      return serverhost;
    }

    public int getServerPort() {
        
      return serverport;   
    }

    public void logout()
    {
      in.logout();
    }
  
    public void openConnection(String hostname, int port) throws IOException {
    
        serverhost = hostname;
       	serverport = port;
        Log.out.println(CL + ": openConnection " + hostname + " " + port, 2);
        initSocket(hostname, port);                 
    }
    
    
    public void closeConnection() throws IOException {
        
//	      serverhost = null;
//	      serverport = 0;
//      reset();	  
        Log.out.println(CL + ": closeConnection", 2);    
	      logout();
    }
     

    //interface for LocalApplications
    //public void receiveFromApp(Message m);
    //public void started(int AID);


    public void sendLAToCS(Message m)
    {
      out.putMessage(m);
    }


    public void started(int AID, int SID)
    {
      in.started(AID, SID);
    }
       

    //interface for DemonProtocol
    //public void sendCSToCAB(Message m);
    //public void sendCSToApp(Message m);


    public void sendCSToCAB(Message m)
    {
      ca.sendCSToCAB(m);                
    }


    public void sendCSToApp(EventMessage m)
    {
      apps.sendCSToApp(m);
    }


    //interface for DemonThreadIn
    //public void connectionBrk();


    public void connectionBrk()
    {
      ca.errorServer(Const.CONNBROKEN);
     //end();
      reset();
    }


    //internal methods
    //private void initSocket();
    //public void end();


    public void initSocket(String hostname, int port) throws IOException  
    {
      Socket s;
      if (serverhost.compareTo("*") == 0)
      {
	hostname = null;
	port = 0;     
	ca.errorServer(Const.FILEWRONGFORMAT);
	return;
      }
      if(port <= 0)
      {
	hostname = null;
	port = 0;      
	ca.errorServer(Const.FILEWRONGFORMAT);
	return;
      }
      if(Const.inNetscape())
	{
	  netscape.security.PrivilegeManager.enablePrivilege("30Capabilities");
	  setNavigatorProxyProperties();
	} else if (Const.inMSIE())
	{
		com.ms.security.PolicyEngine.assertPermission(com.ms.security.PermissionID.SYSTEM); 
	}

//      try
//      {
	s=new Socket(hostname,port);
//      }
//      catch(Exception e)
//      {
//	s = null;
//	System.out.println(CL+".initSocket(): unable to contact server: "+e);
//	e.printStackTrace();
//	ca.errorServer(Const.UNABLETOCONNECT);
//	return;
//     }
      out = new ThreadOut(s);   
      in = new DemonThreadIn(out, s, this);
    }// initSocket _________________________________________________________
 

    public void end()
    {
      //   System.out.println("CentralServer: releasing references"); 
      ca = null;
      apps = null;
      if (in != null)
      {
	in.stop();
	in = null;
      }
      if (out != null)
      {
	out.stop();   
	out = null;
      }
      serverhost = null;
      serverport = 0;
    }
    
    
   private void reset() {
  
      if (in != null) {
              in.stop();
              in = null;
      }
      
      if (out != null) {
              out.stop();   
              out = null;
      }
      
      serverhost = null;
      serverport = 0; 
           
  
  }
 
  public void setNavigatorProxyProperties() {
          
    try {
      PrivilegeManager.enablePrivilege("30Capabilities");
      PrivilegeManager.enablePrivilege("UniversalFileRead");
      String s = System.getProperty("network.proxy.type");
      String platform;
      String unixHome = ".netscape/";
      
      
      if (s != null)
                return;
        /*
        * Search prefs.js for proxy settings
        */
        s = System.getProperty("user.home");
        
        int i, j = s.length();
        if (s.charAt(j-1) != File.separatorChar)
                s += File.separator;
                
        platform = System.getProperty("os.name");
        Log.out.println(CL + ":system " + platform, 2);
        
        if (platform.indexOf("Windows") == -1) {
            
            s = s + unixHome;
        }
        
        Log.out.println(CL + ":looking for prefs.js in " + s, 2);
        
        File f = new File(s + "prefs.js");
        if (!f.exists())
                f = new File(s + "preferences.js");
        BufferedReader in = new BufferedReader(new FileReader(f));
        Properties pref = new Properties();
        String name;
        // Read all preferences into pref
        while ((s = in.readLine()) != null) {
                if (s.indexOf("user_pref") < 0)
                        continue;
                // Parse user_pref("name", "value");
                //    or user_pref("name", 21);
                i = s.indexOf('"') + 1;
                if (i < 1) continue;
                j = s.indexOf('"', i);
                if (j < 0) continue;
                name = s.substring(i, j);
                //
                i = s.indexOf(',', j) + 1;
                if (i < 1) continue;
                j = s.lastIndexOf(')');
                if (j < i) continue;
                s = s.substring(i, j).trim();
                i = s.length() - 1;
                if (i > 0 && s.charAt(0) == '"' && s.charAt(i) == '"')
                        s = s.substring(1, i);
                pref.put(name, s);
        }
        // Get all system properties
        Properties sys = System.getProperties();
        /*
        * network.proxy.type:
        *    null  Direct connection (no proxy)
        *    1     Manual proxy configuration
        *    2     Automatic proxy configuration
        */
        s = pref.getProperty("network.proxy.type");
        if (s == null) {
            s = "0";
            Log.out.println(CL + ": no proxy settings detected", 2);
        }
        
        sys.put("network.proxy.type", s);
        
        if (s.equals("1")) {
                Log.out.println(CL + ": proxy settings detected", 2);
                // Set SOCKS proxy properties
                s = pref.getProperty("network.hosts.socks_server");
                if (s != null) {
                    
                    Log.out.println(CL + ": network.hosts.socks_server: " + s, 2);
                    sys.put("socksProxyHost", s);
                }
                
                s = pref.getProperty("network.hosts.socks_serverport");
                
                if (s != null) {                    
                    Log.out.println(CL + ": network.hosts.socks_serverport: " + s, 2);
                    sys.put("socksProxyPort", s);
                }
                // Set HTTP proxy properties
                s = pref.getProperty("network.proxy.http");
                
                if (s != null) {                    
                    Log.out.println(CL + ": network.proxy.http: " + s, 2);
                    sys.put("http.proxyHost", s);
                }
                
                s = pref.getProperty("network.proxy.http_port");
                
                if (s != null) {                    
                    Log.out.println(CL + ": network.proxy.http_port: " + s, 2);
                    sys.put("http.proxyPort", s);
                }
                
                // Set HTTPS proxy properties
                s = pref.getProperty("network.proxy.ssl");
                if (s != null)  {                    
                    Log.out.println(CL + ": network.proxy.ssl: " + s, 2);
                    sys.put("https.proxyHost", s);
                }
                
                s = pref.getProperty("network.proxy.ssl_port");
                if (s != null)  {                    
                    Log.out.println(CL + ": network.proxy.ssl_port: " + s, 2);
                    sys.put("https.proxyPort", s);
                }
                // Set FTP proxy properties
                s = pref.getProperty("network.proxy.ftp");
                if (s != null) {                    
                   Log.out.println(CL + ": network.proxy.ftp: " + s, 2);
                   sys.put("ftpProxyHost", s);
                }
                s = pref.getProperty("network.proxy.ftp_port");
                if (s != null) {                    
                  Log.out.println(CL + ": network.proxy.ftp_port: " + s, 2);
                  sys.put("ftpProxyPort", s);
                }
        }
        // Make it permanent
        System.setProperties(sys);
    }
    catch (Throwable e) {
        
        e.printStackTrace();    
    }
}



}// CentralServer //////////////////////////////////////////////////////////

