// $Id: Watcher.java,v 1.2 1999/07/26 19:50:44 beca Exp $
/*
  File: Watcher.java
  Component: Plugin
  Date		Author		Changes
  Jun 28 99	Lukasz Beca	Created  
*/

package webwisdom.tango.threads;

import java.net.*;
import java.io.*;
import webwisdom.tango.structures.ControlApplication;
import webwisdom.tango.structures.MessageQueue;
import webwisdom.tango.test.Log;
import webwisdom.tango.messages.WatcherMessage;
import webwisdom.tango.messages.Message;
import webwisdom.tango.messages.WatchNet;
import webwisdom.tango.consts.Const;



public class Watcher {
 
    private ControlApplication ca;
    private WatchThreadIn input;
    private WatchThreadOut output;
    
    public static final byte UNDEFINED = 0;
    public static final byte USER_CONNECTED = 1;
    public static final byte USER_DISCONNECTED = 2;
    public static final byte PING = 3;
     
    public Watcher(ControlApplication control) {
        
        Socket s;        
        ca = control;  
        
        try {            
            netscape.security.PrivilegeManager.enablePrivilege("30Capabilities");
            s = new Socket("localhost", 4403);        
            output = new WatchThreadOut(s);
            input = new WatchThreadIn(s, output);                        
        } catch (Exception e) {
            
            Log.out.println("Watcher: PeopleWatch not installed", 4);
            s = null;
            output = null;
            input = null;            
        }
     }
    
    public void userConnected(String serverHost, int port, String url) {
        
      WatcherMessage m;
      String serverString = serverHost + ":" + port;
      byte serverData[];
      byte urlData[];
      byte combinedData[];
      int length1, length2, totalLength;
      int i;
                  
      serverData = serverString.getBytes();
      urlData = url.getBytes();
      length1 = serverData.length;
      length2 = urlData.length;
            
      totalLength = length1 + length2 + 2;
      
      combinedData = new byte[totalLength];
      
      combinedData[0] = (byte) length1;      
            
      for (i = 0; i < length1; i ++) {
        
        combinedData[i + 1] = serverData[i];         
      }
      
      combinedData[length1 + 1] = (byte) length2;
      
      for (i = 0; i < length2; i ++) {
        
        combinedData[length1 + i + 2] = urlData[i];                 
      }                  
      
      m = new WatcherMessage(USER_CONNECTED, combinedData);
        
      Log.out.println("Watcher: connect" + serverHost + " " + port + " " + url, 2);  
      
      if (output != null) {
      
        output.putMessage(m);        
      }                    
    }
    
    public void userDisconnected() {
      
      byte data[] = new byte[0];
      WatcherMessage m = new WatcherMessage(USER_DISCONNECTED, data);  
      
      
      Log.out.println("Watcher: disconnect", 2);  
      
      if (output != null) {
      
        output.putMessage(m);
      }        
    }           
    
    public void shutdown() {
           
      Log.out.println("Watcher: shutdown", 4);  
      
      if (input != null) {             
      
         input.stop();
      }
      
    }    
                
}




class WatchThreadIn extends WatcherProtocol implements Runnable
{
    private static final String CL="WatchThreadIn";
  

    private WatchNet n;
    private WatchThreadOut out;
    private Thread th;   //reference to this thread 
    private boolean exit;      

    public WatchThreadIn(Socket s, WatchThreadOut outpar)
    {     
      out = outpar;        
      n = new WatchNet(s);   
      th = new Thread(this, "WatchThreadIn:" + hashCode());
      exit = false;
      th.start();
     }// AppThreadIn ________________________________________________________

 
    public void run()
    {
   
      while(!exit)
      {
	WatcherMessage m;
	try
	{
	  m=n.receive();
	}
	catch(IOException e)
	{
	 
	  m=null;	
	  stop();
	  break;
	}
	if(m!=null)
	{
	  handleWATCHEREVENT(m);
	}
	else
	{
	 Log.err.println(CL+": null message received!: ignoring", 2);
	}
      }
    }// run ________________________________________________________________

    
    public void stop()
    {
      //System.out.println(CL+".stop()");
      exit=true;
      out.stop();
    
    }// stop _______________________________________________________________


}// AppThreadIn ////////////////////////////////////////////////////////////


class WatchThreadOut implements Runnable
{
    private static final String CL="WatchThreadOut";


    private MessageQueue q;
    private WatchNet n;
    private Thread th;
    private boolean exit;
    

    static int number=1;    //for testing only
    int thisno;  //for testing only

    private Thread pinger;


    public WatchThreadOut(Socket s)
    {
      super();    

      thisno=number;
      number++;

      n=new WatchNet(s);
      q=new MessageQueue();
      th=new Thread(this, "WatchThreadOut:" + hashCode());
      exit=false;
      th.start();

      pinger=new Thread(new WatchPinger(n,600000), "Pinger:" + hashCode());
      pinger.start();
    }// AppThreadOut _______________________________________________________


    public void run()
    {
      Log.out.println(CL+".run(): "+thisno, 2);
      while(!exit)
      {
	Message m=q.getMessage();
	try
	{
	  n.send(m);
          //System.out.println(CL+": "+thisno+" sending "+m);
	}
	catch(IOException e)
	{
	  Log.err.println(CL+": error sending message: "+e, 2);
	  e.printStackTrace(Log.err);
	}
      }
    }// run ________________________________________________________________


    public synchronized void putMessage(Message m)
    {
      q.putMessage(m);
    }// putMessage _________________________________________________________

 
    public void stop()
    {
      //System.out.println(CL+".stop()");
      exit = true;
      q.unblock();
      pinger.stop();
      n.close();
    }// stop _______________________________________________________________


}




class WatchPinger implements Runnable
{
    private static final String CL="Pinger";
    
    private Thread _th;
    private int _interval;
    private WatchNet n;


    public WatchPinger(WatchNet npar,int time)
    {
      n=npar;
      _interval=time;
    }// Pinger ____________________________________________________________
    

    public void run()
    {
      byte[] data=new byte[0];     
      WatcherMessage pm=new WatcherMessage(Watcher.PING, data);

      //System.out.println(CL+".run()");
      while(true)
      {
	try
	{
	  Thread.sleep(_interval);
	}
	catch(InterruptedException e)
	{
	  Log.err.println(CL+": interruption: "+e, 2);
	  e.printStackTrace(Log.err);
	}
	try
	{
	  n.send(pm);
	  //System.out.println(CL+": sent");
	}
	catch(IOException e)
	{
	  Log.err.println(CL+": error sending message: "+e, 2);
	  e.printStackTrace(Log.err);
	}
      }
    }// run ________________________________________________________________


}// Pinger /////////////////////////////////////////////////////////////////


class WatcherProtocol
{
    private static final String CL="WatcherProtocol";        
      
    public void handleWATCHEREVENT(WatcherMessage m)
    {
      //lapps.sendAppToOther(AID,m);
      
      Log.out.println(CL + ": received message from PeopleWatch", 4);
      
    }// handleAPPEVENT _____________________________________________________


}// ApplicationProtocol ////////////////////////////////////////////////////